package com.sunxung.factoring.service.financing.review;

import com.baomidou.mybatisplus.extension.service.IService;
import com.sunxung.factoring.entity.financing.review.ReviewFinancingInfo;
import com.sunxung.factoring.entity.financing.review.ReviewIncomeExpenditurePlan;
import com.sunxung.factoring.entity.financing.review.ReviewQuotaDisburse;
import com.sunxung.factoring.entity.financing.review.ReviewRltRepay;
import com.sunxung.factoring.entity.smb.FinancingFromInvestorsNotice;
import com.sunxung.factoring.entity.sys.Dictionary;

import java.util.List;

/**
 * @Description : 额度支用service
 * @Author : L.Jia
 * @Date: 2024-01-08 09:29
 */
public interface ReviewQuotaDisburseService extends IService<ReviewQuotaDisburse> {

    /**
     * 根据融资信息生成收支计划
     *
     * @param reviewFinancingInfo
     * @return
     */
    List<ReviewIncomeExpenditurePlan> generateIncomeExpenditurePlan(ReviewFinancingInfo reviewFinancingInfo);


    /**
     * 生成苏商银行还款计划
     *
     * @param reviewFinancingInfo
     * @param planList
     * @return
     */
    List<ReviewIncomeExpenditurePlan> generateSMBPlan(ReviewFinancingInfo reviewFinancingInfo, List<ReviewIncomeExpenditurePlan> planList);

    /**
     * 根据收支计划信息：计算IRR
     *
     * @param quotaDisburse
     * @return
     */
    String calculateIrr(ReviewQuotaDisburse quotaDisburse);

    /**
     * 获取相关还款责任人类型
     *
     * @param code
     * @return
     */
    List<Dictionary> findArlpType(String code);

    /**
     * 新增额度支用信息
     *
     * @param reviewQuotaDisburse 额度支用实体类
     * @return ReviewQuotaDisburse 额度支用实体类
     */
    void editReviewQuotaDisburse(ReviewQuotaDisburse reviewQuotaDisburse) throws Exception;

    /**
     * 根据供应商获取资方授信信息
     *
     * @param financingBasicInfoId
     * @return
     */
    FinancingFromInvestorsNotice getFinancingFromInvestorsNotice(Long financingBasicInfoId);

    /**
     * 根据供应商获取资方授信法人信息
     *
     * @param financingBasicInfoId
     * @return
     */
    List<ReviewRltRepay> getFinancingFromInvestorsPerson(Long financingBasicInfoId, String arlpAmt);
}
