package com.sunxung.factoring.service.smb.impl;

import com.alibaba.fastjson.JSONObject;
import com.sunxung.factoring.component.util.ElectronicSignatureUtil;
import com.sunxung.factoring.component.util.StringUtil;
import com.sunxung.factoring.dict.impl.FileModuleDict;
import com.sunxung.factoring.entity.acceptance.AcceptanceSignConfirmationElectronicInfo;
import com.sunxung.factoring.entity.customerinformation.ElectronicSignaturePersonal;
import com.sunxung.factoring.entity.smb.FinancingFromInvestorsEletronicContract;
import com.sunxung.factoring.entity.sys.FileStorage;
import com.sunxung.factoring.mapper.smb.FinancingFromInvestorsEletronicContractMapper;
import com.sunxung.factoring.service.customerinformation.IElectronicSignaturePersonalService;
import com.sunxung.factoring.service.smb.IFinancingFromInvestorsEletronicContractService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sunxung.factoring.service.sys.FileService;
import com.sunxung.factoring.service.sys.FileStorageService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 资方融资管理-电子签信息 服务实现类
 * </p>
 *
 * @author liutao
 * @since 2024-08-28
 */
@Service
public class FinancingFromInvestorsEletronicContractServiceImpl extends ServiceImpl<FinancingFromInvestorsEletronicContractMapper, FinancingFromInvestorsEletronicContract> implements IFinancingFromInvestorsEletronicContractService {

    @Autowired
    private FileStorageService fileStorageService;

    @Autowired
    private IElectronicSignaturePersonalService electronicSignaturePersonalService;

    @Override
    public void doSignCallBack(FinancingFromInvestorsEletronicContract financingFromInvestorsEletronicContract) {
        String contractId = financingFromInvestorsEletronicContract.getContractId();

        //合同下载
        String contDownUrl = ElectronicSignatureUtil.downloadContract(contractId);
        FileStorage backFile = fileStorageService.addFileStorageWithBelongAndName(contractId,
                FileModuleDict.ChildEnum.SMB_SING_BACK_FILE, contDownUrl, null);
        fileStorageService.saveAttachmentRefByFileStorage(backFile,
                FileModuleDict.ChildEnum.SMB_SING_BACK_FILE,
                financingFromInvestorsEletronicContract.getId());
        //合同归档
        ElectronicSignatureUtil.contractFiled(contractId);

        //电子文件签署线上出证下载
        String reportDownUrl = ElectronicSignatureUtil.complianceContractReport(contractId);
        FileStorage reportFile = fileStorageService.addFileStorageWithBelongAndName(contractId,
                FileModuleDict.ChildEnum.SMB_SING_REPORT_FILE, reportDownUrl, null);
        fileStorageService.saveAttachmentRefByFileStorage(reportFile,
                FileModuleDict.ChildEnum.SMB_SING_REPORT_FILE,
                financingFromInvestorsEletronicContract.getId());

        financingFromInvestorsEletronicContract.setSignStatus(true);
        financingFromInvestorsEletronicContract.setBackFileId(backFile.getId());
        financingFromInvestorsEletronicContract.setReportFileId(reportFile.getId());

        updateById(financingFromInvestorsEletronicContract);
    }
}
