package com.sunxung.factoring.service.smb.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.deepoove.poi.XWPFTemplate;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.snb.fsos.bean.AccountInfoQuery;
import com.snb.fsos.bean.ProductCreditApplyV2;
import com.snb.fsos.bean.SMBFileUpload;
import com.sunxung.factoring.component.enums.CreditInvestigationTypeEnums;
import com.sunxung.factoring.component.enums.SmbCreditStatusEnum;
import com.sunxung.factoring.component.enums.SmbLimitStatusEnum;
import com.sunxung.factoring.component.enums.SupplierStatusEnums;
import com.sunxung.factoring.component.exception.BusinessException;
import com.sunxung.factoring.component.exception.ValidatorException;
import com.sunxung.factoring.component.util.*;
import com.sunxung.factoring.component.util.onlineConverter.OfficeConverterUtil;
import com.sunxung.factoring.dict.impl.FileModuleDict;
import com.sunxung.factoring.entity.GridPage;
import com.sunxung.factoring.entity.customerinformation.ElectronicSignaturePersonal;
import com.sunxung.factoring.entity.electronicsignature.FddSignData;
import com.sunxung.factoring.entity.entprise.Enterprise;
import com.sunxung.factoring.entity.financing.contractTemplate.ContractTemplate;
import com.sunxung.factoring.entity.project.BusinessProcessingLog;
import com.sunxung.factoring.entity.smb.*;
import com.sunxung.factoring.entity.smb.vo.FinancingFromInvestorsSearchVo;
import com.sunxung.factoring.entity.supplier.*;
import com.sunxung.factoring.entity.sys.*;
import com.sunxung.factoring.entity.sys.Dictionary;
import com.sunxung.factoring.mapper.smb.FinancingFromInvestorsMapper;
import com.sunxung.factoring.service.customerinformation.IElectronicSignaturePersonalService;
import com.sunxung.factoring.service.entprise.IEnterpriseService;
import com.sunxung.factoring.service.financing.contractTemplate.IContractTemplateService;
import com.sunxung.factoring.service.project.BusinessProcessingLogService;
import com.sunxung.factoring.service.smb.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sunxung.factoring.service.smbApi.api.SMBApiService;
import com.sunxung.factoring.service.smbApi.dto.CreditApplyRiskMsgDto;
import com.sunxung.factoring.service.smbApi.dto.LoanCreditRiskMsgDto;
import com.sunxung.factoring.service.smbApi.resp.AccountInfoQueryResp;
import com.sunxung.factoring.service.smbApi.resp.CreditApplyResp;
import com.sunxung.factoring.service.smbApi.resp.CreditProcessQueryResp;
import com.sunxung.factoring.service.supplier.*;
import com.sunxung.factoring.service.sys.*;
import com.sunxung.factoring.service.sys.dto.AttachmentDto;
import org.apache.commons.io.FileUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 资方融资管理 服务实现类
 * </p>
 *
 * @author liutao
 * @since 2024-08-26
 */
@Service
public class FinancingFromInvestorsServiceImpl extends ServiceImpl<FinancingFromInvestorsMapper, FinancingFromInvestors> implements IFinancingFromInvestorsService {

    private static Logger log = LogManager.getLogger(IFinancingFromInvestorsService.class);

    @Autowired
    private IFinancingFromInvestorsPersonService financingFromInvestorsPersonService;

    @Autowired
    private IFinancingFromInvestorsContractService financingFromInvestorsContractService;

    @Autowired
    private IFinancingFromInvestorsHistoryService financingFromInvestorsHistoryService;

    @Autowired
    private IFinancingFromInvestorsPersonHistoryService financingFromInvestorsPersonHistoryService;

    @Autowired
    private IFinancingFromInvestorsContractHistoryService financingFromInvestorsContractHistoryService;

    @Autowired
    private FileService fileService;

    @Autowired
    private BusinessProcessingLogService businessProcessingLogService;

    @Autowired
    private ICSupplierInfoHistoryService supplierInfoHistoryService;

    @Autowired
    private ICEnterpriseHistoryService enterpriseHistoryService;

    @Autowired
    private ISupplierInfoService supplierInfoService;

    @Autowired
    private IEnterpriseService enterpriseService;

    @Autowired
    private IElectronicSignaturePersonalService electronicSignaturePersonalService;

    @Autowired
    private ICreditInvestigationInfoService creditInvestigationInfoService;

    @Autowired
    private IContractTemplateService contractTemplateService;

    @Autowired
    private ISysAttachmentRefService sysAttachmentRefService;

    @Autowired
    private IFinancingFromInvestorsEletronicContractService financingFromInvestorsEletronicContractService;

    @Autowired
    private SMBApiService smbApiService;

    @Autowired
    private UserService userService;

    @Autowired
    private RoleService roleService;

    @Autowired
    private PermissionService permissionService;

    @Autowired
    private IFinancingFromInvestorsNoticeService financingFromInvestorsNoticeService;

    @Autowired
    private FileStorageService fileStorageService;

    @Autowired
    private ICSalesDocumentsInfoHistoryService salesDocumentsInfoHistoryService;

    @Autowired
    private DictionaryService dictionaryService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addInfo(FinancingFromInvestors financingFromInvestors) {
        //续授信
        if(financingFromInvestors.getCredit()){
            //判断数据的状态是否支持续授信
            List<FinancingFromInvestorsHistory> financingFromInvestorsHistories = financingFromInvestorsHistoryService.lambdaQuery().eq(FinancingFromInvestorsHistory::getFinancingFromInvestorsId, financingFromInvestors.getId()).orderByDesc(FinancingFromInvestorsHistory::getId).list();
            FinancingFromInvestorsNotice financingFromInvestorsNotice = null;
            if (CollectionUtil.isNotEmpty(financingFromInvestorsHistories)) {
                FinancingFromInvestorsHistory financingFromInvestorsHistory = financingFromInvestorsHistories.get(0);
                financingFromInvestorsNotice = financingFromInvestorsNoticeService.lambdaQuery().eq(FinancingFromInvestorsNotice::getFinancingFromInvestorsHistoryId, financingFromInvestorsHistory.getId()).one();
            }
            if(financingFromInvestorsNotice == null){
                throw new BusinessException(CodeUtil.FAIL,"无法续授信！请联系开发人员！");
            }
            if (financingFromInvestorsNotice.getExpiringDate().compareTo(new Date()) == 1) {
                throw new BusinessException(CodeUtil.FAIL, "授信生效中，无法续授信！");
            }
            if (financingFromInvestorsNotice.getNextApplyDate().compareTo(new Date()) == 1) {
                throw new BusinessException(CodeUtil.FAIL, "未到可再次申请日期，无法续授信！");
            }
            financingFromInvestors.setId(null);
            financingFromInvestors.setCreditStatus(null);
            financingFromInvestors.setRepaymentAccount(null);
            financingFromInvestors.setRepaymentAccountName(null);
            financingFromInvestors.setRepaymentAccountType(null);
        }
        List<SMBFileUpload> fileList = new ArrayList<>();
        List<Long> registeredAddressCodeList = financingFromInvestors.getRegisteredAddressCodeList();
        if(CollectionUtil.isNotEmpty(registeredAddressCodeList)){
            String address = registeredAddressCodeList.stream().map(r -> String.valueOf(r)).collect(Collectors.joining(","));
            financingFromInvestors.setRegisteredAddress(address);
        }
        //基础信息
        this.saveOrUpdate(financingFromInvestors);
        SupplierInfo supplierInfo = supplierInfoService.get(financingFromInvestors.getSupplierInfoId());
        //企业信息
        Enterprise enterprise = enterpriseService.getById(supplierInfo.getcEnterpriseId());
        Long fileId = financingFromInvestors.getFileId();
        if (fileId != null) {
            FileStorageDO file = fileService.getById(fileId);
            file.setId(null);
            file.setBusinessId(financingFromInvestors.getId());
            file.setBusinessType(FileModuleDict.ChildEnum.SMB_BUSINESS_LICENSE_FILE.getCode());
            fileService.save(file);
            //苏商文件上传参数 索引0：营业执照
            SMBFileUpload smbFileUpload = new SMBFileUpload();
            smbFileUpload.setFileName(file.getOriginalName());
            smbFileUpload.setFileType("306");
            smbFileUpload.setFilePath(file.getAbsolutePath());
            fileList.add(smbFileUpload);
        }
        //法人信息
        financingFromInvestorsPersonService.lambdaUpdate().eq(FinancingFromInvestorsPerson::getFinancingFromInvestorsId, financingFromInvestors.getId()).remove();
        FinancingFromInvestorsPerson financingFromInvestorsPerson = financingFromInvestors.getFinancingFromInvestorsPerson();
        if (financingFromInvestorsPerson != null) {
            financingFromInvestorsPerson.setFinancingFromInvestorsId(financingFromInvestors.getId());
            financingFromInvestorsPersonService.save(financingFromInvestorsPerson);
            Long frontId = financingFromInvestorsPerson.getFrontId();
            if (frontId != null) {
                FileStorageDO front = fileService.getById(frontId);
                front.setId(null);
                front.setBusinessId(financingFromInvestorsPerson.getId());
                front.setBusinessType(FileModuleDict.ChildEnum.SMB_PERSON_FRONT_FILE.getCode());
                fileService.save(front);
                //苏商文件上传参数 索引1：身份证正面
                SMBFileUpload smbFileUpload = new SMBFileUpload();
                smbFileUpload.setFileName(front.getOriginalName());
                smbFileUpload.setFileType("304");
                smbFileUpload.setFilePath(front.getAbsolutePath());
                fileList.add(smbFileUpload);
            }
            Long backId = financingFromInvestorsPerson.getBackId();
            if (backId != null) {
                FileStorageDO back = fileService.getById(backId);
                back.setId(null);
                back.setBusinessId(financingFromInvestorsPerson.getId());
                back.setBusinessType(FileModuleDict.ChildEnum.SMB_PERSON_BACK_FILE.getCode());
                fileService.save(back);
                //苏商文件上传参数 索引2：身份证反面
                SMBFileUpload smbFileUpload = new SMBFileUpload();
                smbFileUpload.setFileName(back.getOriginalName());
                smbFileUpload.setFileType("305");
                smbFileUpload.setFilePath(back.getAbsolutePath());
                fileList.add(smbFileUpload);
            }
        }
        //合同文件
        List<String> fileInfos = new ArrayList<>();
        List<CreditApplyRiskMsgDto.ContractInfo> contractInfos = new ArrayList<>();
        financingFromInvestorsContractService.lambdaUpdate().eq(FinancingFromInvestorsContract::getFinancingFromInvestorsId, financingFromInvestors.getId()).remove();
        List<FinancingFromInvestorsContract> financingFromInvestorsContracts = financingFromInvestors.getFinancingFromInvestorsContracts();
        if (CollectionUtil.isNotEmpty(financingFromInvestorsContracts)) {
            for (FinancingFromInvestorsContract financingFromInvestorsContract : financingFromInvestorsContracts) {
                financingFromInvestorsContract.setFinancingFromInvestorsId(financingFromInvestors.getId());
                financingFromInvestorsContractService.save(financingFromInvestorsContract);
                //苏商合同信息
                CreditApplyRiskMsgDto.ContractInfo contractInfo = new CreditApplyRiskMsgDto.ContractInfo();
                contractInfo.setContractName(financingFromInvestorsContract.getContractName());
                contractInfo.setContractor(financingFromInvestorsContract.getName());
                contractInfo.setContractAmount(financingFromInvestorsContract.getContractAmount());
                contractInfo.setContractNo(financingFromInvestorsContract.getContractNumber());
                contractInfo.setContractTime(DateUtil.formatDate(financingFromInvestorsContract.getContractDate()).replace("-",""));
                contractInfos.add(contractInfo);
                Long file = financingFromInvestorsContract.getFileId();
                if (file != null) {
                    FileStorageDO fileStorageDO = fileService.getById(file);
                    fileStorageDO.setId(null);
                    fileStorageDO.setBusinessId(financingFromInvestorsContract.getId());
                    fileStorageDO.setBusinessType(FileModuleDict.ChildEnum.SMB_CONTRACT_FILE.getCode());
                    fileService.save(fileStorageDO);
                    //苏商合同文件
                    String fileSdossId ;
                    try {
                        List<SMBFileUpload> upload = new ArrayList<>();
                        SMBFileUpload smbFileUpload = new SMBFileUpload();
                        smbFileUpload.setFileName(fileStorageDO.getOriginalName());
                        smbFileUpload.setFileType("924");
                        smbFileUpload.setFilePath(fileStorageDO.getAbsolutePath());
                        upload.add(smbFileUpload);
                        Map<String, String> map = smbApiService.fileStreamUpload(upload);
                        fileSdossId = map.get(fileStorageDO.getOriginalName());
                    } catch (Exception e) {
                        throw new BusinessException(CodeUtil.FAIL, "上传苏商文件出错，请稍后重试！");
                    }
                    fileInfos.add(fileSdossId);
                }
            }
        }
        if (financingFromInvestors.getVisible()) {
            //判断是否所有合同签署完成
            List<FinancingFromInvestorsEletronicContract> list = financingFromInvestorsEletronicContractService.lambdaQuery().eq(FinancingFromInvestorsEletronicContract::getcEnterpriseId, enterprise.getId()).list();
            if (CollectionUtil.isNotEmpty(list)) {
                for (FinancingFromInvestorsEletronicContract financingFromInvestorsEletronicContract : list) {
                    if (!financingFromInvestorsEletronicContract.getSignStatus()) {
                        throw new BusinessException(CodeUtil.FAIL, "请完成所有合同的签署后提交！");
                    }
                }
            }
            FinancingFromInvestorsEletronicContract financingFromInvestorsEletronicContractOne = financingFromInvestorsEletronicContractService
                    .lambdaQuery().eq(FinancingFromInvestorsEletronicContract::getcEnterpriseId, enterprise.getId()).eq(FinancingFromInvestorsEletronicContract::getContractType, 0).one();
            FileStorage one = fileStorageService.get(financingFromInvestorsEletronicContractOne.getBackFileId());
            //苏商文件上传参数 索引3：企业征信查询授权书
            SMBFileUpload smbFileUpload = new SMBFileUpload();
            smbFileUpload.setFileName(one.getOriginalName());
            smbFileUpload.setFileType("302");
            smbFileUpload.setFilePath(one.getAbsolutePath());
            fileList.add(smbFileUpload);
            FinancingFromInvestorsEletronicContract financingFromInvestorsEletronicContractTwo = financingFromInvestorsEletronicContractService
                    .lambdaQuery().eq(FinancingFromInvestorsEletronicContract::getcEnterpriseId, enterprise.getId()).eq(FinancingFromInvestorsEletronicContract::getContractType, 1).one();
            FileStorage two = fileStorageService.get(financingFromInvestorsEletronicContractTwo.getBackFileId());
            //苏商文件上传参数 索引4：个人征信查询授权书
            SMBFileUpload smbFileUploadTwo = new SMBFileUpload();
            smbFileUploadTwo.setFileName(two.getOriginalName());
            smbFileUploadTwo.setFileType("301");
            smbFileUploadTwo.setFilePath(two.getAbsolutePath());
            fileList.add(smbFileUploadTwo);
            FinancingFromInvestorsEletronicContract financingFromInvestorsEletronicContractThree = financingFromInvestorsEletronicContractService
                    .lambdaQuery().eq(FinancingFromInvestorsEletronicContract::getcEnterpriseId, enterprise.getId()).eq(FinancingFromInvestorsEletronicContract::getContractType, 2).one();
            FileStorage three = fileStorageService.get(financingFromInvestorsEletronicContractThree.getBackFileId());
            //苏商文件上传参数 索引5：个人综合信息查询授权书
            SMBFileUpload smbFileUploadThree = new SMBFileUpload();
            smbFileUploadThree.setFileName(three.getOriginalName());
            smbFileUploadThree.setFileType("303");
            smbFileUploadThree.setFilePath(three.getAbsolutePath());
            fileList.add(smbFileUploadThree);
            FileStorage face = fileStorageService.get(financingFromInvestorsEletronicContractThree.getFaceFileId());
            //苏商文件上传参数 索引6：法人人脸识别照片
            SMBFileUpload smbFileUploadface = new SMBFileUpload();
            smbFileUploadface.setFileName(face.getOriginalName());
            smbFileUploadface.setFileType("308");
            smbFileUploadface.setFilePath(face.getAbsolutePath());
            fileList.add(smbFileUploadface);
            //调苏商银行接口
            ProductCreditApplyV2 creditApply = new ProductCreditApplyV2();
            creditApply.setCustType("1");
            creditApply.setCustName(enterprise.getCorporateName());
            creditApply.setCertType("20");
            creditApply.setCertNo(enterprise.getSocialCreditCode());
            creditApply.setPhoneNo(financingFromInvestorsPerson.getPhoneNumber());
            creditApply.setLegalPersonName(financingFromInvestorsPerson.getName());
            creditApply.setLegalPersonCertType("10");
            creditApply.setLegalPersonCertNo(financingFromInvestorsPerson.getSocialCreditCode());
            String bizNo = UUID.randomUUID().toString().replace("-","");
            if(bizNo.length() > 32){
                bizNo = bizNo.substring(0,32);
            }
            creditApply.setBizNo(bizNo);
            CreditApplyRiskMsgDto creditApplyRiskMsg = new CreditApplyRiskMsgDto();
            for (int i = 0; i < fileList.size(); i++) {
                SMBFileUpload fileUpload = fileList.get(i);
                List<SMBFileUpload> param = new ArrayList<>();
                param.add(fileUpload);
                String value;
                try {
                    Map<String, String> stringStringMap = smbApiService.fileStreamUpload(param);
                    value = stringStringMap.get(fileUpload.getFileName());
                } catch (Exception e) {
                    throw new BusinessException(CodeUtil.FAIL, "上传苏商文件出错，请稍后重试！");
                }
                if (i == 0) {
                    creditApplyRiskMsg.setType306SdossId(value);
                } else if (i == 1) {
                    creditApplyRiskMsg.setType304SdossId(value);
                } else if (i == 2) {
                    creditApplyRiskMsg.setType305SdossId(value);
                } else if (i == 3) {
                    creditApplyRiskMsg.setType302SdossId(value);
                } else if (i == 4) {
                    creditApplyRiskMsg.setType301SdossId(value);
                } else if (i == 5) {
                    creditApplyRiskMsg.setType303SdossId(value);
                } else if (i == 6) {
                    creditApplyRiskMsg.setType308SdossId(value);
                }
            }
            creditApplyRiskMsg.setType924SdossId(fileInfos);
            creditApplyRiskMsg.setCyData(contractInfos);
            creditApplyRiskMsg.setBasicAccount(financingFromInvestors.getBankAccount());
            creditApplyRiskMsg.setBasicAccountBank(financingFromInvestors.getBank());
            creditApplyRiskMsg.setRegCapital(new BigDecimal(financingFromInvestors.getRegisteredCapital().replace("万元人民币","")).multiply(new BigDecimal(10000)));
            creditApplyRiskMsg.setPaidUpCapital(financingFromInvestors.getOperatingRevenue());
            creditApplyRiskMsg.setBusiScope(enterprise.getBusinessScope());
            creditApplyRiskMsg.setWorkers(financingFromInvestors.getPeopleEngaged().toString());
            creditApplyRiskMsg.setRegAddress(financingFromInvestors.getRegisteredAddress());
            creditApplyRiskMsg.setIdCardAddress(financingFromInvestorsPerson.getDomicileAddress());
            creditApplyRiskMsg.setCurrResidenceAddress(financingFromInvestorsPerson.getPermanentAddress());
            creditApplyRiskMsg.setBusiAddress(financingFromInvestors.getBusinessAddress());
            creditApplyRiskMsg.setLegalPersonStartDate(DateUtil.formatDate(financingFromInvestorsPerson.getValidFrom()).replace("-",""));
            creditApplyRiskMsg.setLegalPersonEndDate(DateUtil.formatDate(financingFromInvestorsPerson.getValidTo()).replace("-",""));
            creditApplyRiskMsg.setIndustry(financingFromInvestors.getIndustryTypeFore());
            creditApplyRiskMsg.setEntFoundDate(DateUtil.formatDate(enterprise.getStartSate()).replace("-",""));
            log.info("苏商银行授信请求：" + creditApply.toString() + creditApplyRiskMsg.toString());
            CreditApplyResp creditApplyResp = smbApiService.creditApply(creditApply, creditApplyRiskMsg);
            log.info("苏商银行授信请求結果：" + creditApplyResp.toString());
            // 状态 01 待审批；02 审批中；03 审批通过；04 审批拒绝；06 已归档；07 处理中
            if (creditApplyResp.getStatus().equals("01")) {
                financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_01.getCode());
            } else if (creditApplyResp.getStatus().equals("02")) {
                financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_02.getCode());
            } else if (creditApplyResp.getStatus().equals("03")) {
                financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_03.getCode());
            } else if (creditApplyResp.getStatus().equals("04")) {
                financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_04.getCode());
            } else if (creditApplyResp.getStatus().equals("06")) {
                financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_06.getCode());
            } else if (creditApplyResp.getStatus().equals("07")) {
                financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_07.getCode());
            }
            financingFromInvestors.setOpenId(creditApplyResp.getOpenId());
            //保存历史
            addHistory(financingFromInvestors, bizNo);
            this.updateById(financingFromInvestors);
        }
    }

    private void addHistory(FinancingFromInvestors financingFromInvestors, String bizNo) {
        //基础信息
        FinancingFromInvestorsHistory financingFromInvestorsHistory = BeanUtilsProxy.copyProperties(financingFromInvestors, FinancingFromInvestorsHistory.class);
        financingFromInvestorsHistory.setId(null);
        financingFromInvestorsHistory.setBizNo(bizNo);
        financingFromInvestorsHistory.setFinancingFromInvestorsId(financingFromInvestors.getId());
        financingFromInvestorsHistoryService.save(financingFromInvestorsHistory);
        Long fileId = financingFromInvestors.getFileId();
        if (fileId != null) {
            FileStorageDO file = fileService.getById(fileId);
            file.setId(null);
            file.setBusinessId(financingFromInvestorsHistory.getId());
            file.setBusinessType(FileModuleDict.ChildEnum.SMB_BUSINESS_LICENSE_FILE_HISTORY.getCode());
            fileService.save(file);
        }
        //法人信息
        FinancingFromInvestorsPerson financingFromInvestorsPerson = financingFromInvestors.getFinancingFromInvestorsPerson();
        if (financingFromInvestorsPerson != null) {
            FinancingFromInvestorsPersonHistory financingFromInvestorsPersonHistory = BeanUtilsProxy.copyProperties(financingFromInvestorsPerson, FinancingFromInvestorsPersonHistory.class);
            financingFromInvestorsPersonHistory.setId(null);
            financingFromInvestorsPersonHistory.setFinancingFromInvestorsId(financingFromInvestorsHistory.getId());
            financingFromInvestorsPersonHistoryService.save(financingFromInvestorsPersonHistory);
            Long frontId = financingFromInvestorsPerson.getFrontId();
            if (frontId != null) {
                FileStorageDO front = fileService.getById(frontId);
                front.setId(null);
                front.setBusinessId(financingFromInvestorsPersonHistory.getId());
                front.setBusinessType(FileModuleDict.ChildEnum.SMB_PERSON_FRONT_FILE_HISTORY.getCode());
                fileService.save(front);
            }
            Long backId = financingFromInvestorsPerson.getBackId();
            if (backId != null) {
                FileStorageDO back = fileService.getById(backId);
                back.setId(null);
                back.setBusinessId(financingFromInvestorsPersonHistory.getId());
                back.setBusinessType(FileModuleDict.ChildEnum.SMB_PERSON_BACK_FILE_HISTORY.getCode());
                fileService.save(back);
            }
        }
        //合同文件
        List<FinancingFromInvestorsContract> financingFromInvestorsContracts = financingFromInvestors.getFinancingFromInvestorsContracts();
        if (CollectionUtil.isNotEmpty(financingFromInvestorsContracts)) {
            for (FinancingFromInvestorsContract financingFromInvestorsContract : financingFromInvestorsContracts) {
                FinancingFromInvestorsContractHistory financingFromInvestorsContractHistory = BeanUtilsProxy.copyProperties(financingFromInvestorsContract, FinancingFromInvestorsContractHistory.class);
                financingFromInvestorsContractHistory.setId(null);
                financingFromInvestorsContractHistory.setFinancingFromInvestorsId(financingFromInvestorsHistory.getId());
                financingFromInvestorsContractHistoryService.save(financingFromInvestorsContractHistory);
                Long file = financingFromInvestorsContract.getFileId();
                if (file != null) {
                    FileStorageDO fileStorageDO = fileService.getById(file);
                    fileStorageDO.setId(null);
                    fileStorageDO.setBusinessId(financingFromInvestorsContractHistory.getId());
                    fileStorageDO.setBusinessType(FileModuleDict.ChildEnum.SMB_CONTRACT_FILE_HISTORY.getCode());
                    fileService.save(fileStorageDO);
                }
            }
        }
    }

    @Override
    public List<SupplierInfo> getSuppliers() {
        Long userId = ThreadLocalUtil.getUserId();
        List<SupplierInfo> list = supplierInfoService.getcreditSuccess(userId, SupplierStatusEnums.CREDIT_SUCCESS.getCode());
        if (CollectionUtil.isNotEmpty(list)) {
            for (SupplierInfo supplierInfo : list) {
                Enterprise enterprise = enterpriseService.getById(supplierInfo.getcEnterpriseId());
                List<FileStorageDO> businessLicense = fileService.findByBusinessId(enterprise.getId(), FileModuleDict.ChildEnum.ENTERPRISE_BUSINESS_LICENSE.getCode());
                enterprise.setBusinessLicense(businessLicense);

                CreditInvestigationInfo creditInvestigationInfo = creditInvestigationInfoService.lambdaQuery().eq(CreditInvestigationInfo::getcEnterpriseId, enterprise.getId()).eq(CreditInvestigationInfo::getType, CreditInvestigationTypeEnums.ONE.getCode()).one();
                FileStorageDO front = fileService.findOneByBusinessId(creditInvestigationInfo.getId(), FileModuleDict.ChildEnum.ENTERPRISE_IDCARD_FRONT.getCode());
                FileStorageDO back = fileService.findOneByBusinessId(creditInvestigationInfo.getId(), FileModuleDict.ChildEnum.ENTERPRISE_IDCARD_BACK.getCode());
                creditInvestigationInfo.setFront(front);
                creditInvestigationInfo.setBack(back);
                enterprise.setCreditInvestigationInfo(creditInvestigationInfo);
                supplierInfo.setEnterprise(enterprise);

                BusinessProcessingLog lastLog = businessProcessingLogService.getLastByBusinessKeyBusinessTypeTaskName(supplierInfo.getId(), ConstantUtil.SUPPLIER_INFO, "供应商C端录入");
                CSupplierInfoHistory supplierInfoHistory = supplierInfoHistoryService.getById(lastLog.getFormBusinessId());
                List<CSalesDocumentsInfoHistory> cSalesDocumentsInfoHistories = salesDocumentsInfoHistoryService.lambdaQuery().eq(CSalesDocumentsInfoHistory::getcEnterpriseHistoryId, supplierInfoHistory.getcEnterpriseHistoryId()).list();
                if(CollectionUtil.isNotEmpty(cSalesDocumentsInfoHistories)){
                    for (CSalesDocumentsInfoHistory cSalesDocumentsInfoHistory : cSalesDocumentsInfoHistories) {
                        List<FileStorageDO> fileStorageDOS = fileService.findByBusinessId(cSalesDocumentsInfoHistory.getId(), FileModuleDict.ChildEnum.ENTERPRISE_SALES_DOCUMENTS_HISTORY.getCode());
                        cSalesDocumentsInfoHistory.setFileStorages(fileStorageDOS);
                    }
                }
                supplierInfo.setcSalesDocumentsInfoHistories(cSalesDocumentsInfoHistories);
            }
        }
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<FinancingFromInvestorsEletronicContract> makeContract(Long supplierId) {
        SupplierInfo supplierInfo = supplierInfoService.get(supplierId);
        //企业信息
        Enterprise enterprise = enterpriseService.getById(supplierInfo.getcEnterpriseId());
        List<FinancingFromInvestorsEletronicContract> list = financingFromInvestorsEletronicContractService.lambdaQuery().eq(FinancingFromInvestorsEletronicContract::getcEnterpriseId, enterprise.getId()).list();
        if (CollectionUtil.isEmpty(list)) {
            list = new ArrayList<>();
            //法人信息
            CreditInvestigationInfo creditInvestigationInfo = creditInvestigationInfoService.lambdaQuery().eq(CreditInvestigationInfo::getcEnterpriseId, supplierInfo.getcEnterpriseId())
                    .eq(CreditInvestigationInfo::getType, CreditInvestigationTypeEnums.ONE.getCode()).one();
            ElectronicSignaturePersonal signaturePersonal = electronicSignaturePersonalService.lambdaQuery().eq(ElectronicSignaturePersonal::getName, creditInvestigationInfo.getName())
                    .eq(ElectronicSignaturePersonal::getSocialCreditCode, creditInvestigationInfo.getSocialCreditCode()).one();
            if (signaturePersonal == null) {
                signaturePersonal = new ElectronicSignaturePersonal();
                signaturePersonal.setName(creditInvestigationInfo.getName());
                signaturePersonal.setCardType("idCard");
                signaturePersonal.setMobile(creditInvestigationInfo.getPhoneNumber());
                signaturePersonal.setSocialCreditCode(creditInvestigationInfo.getSocialCreditCode());
                electronicSignaturePersonalService.getAuthInfo(signaturePersonal);
                electronicSignaturePersonalService.save(signaturePersonal);
            }
            ContractTemplate powerOfAttorneyCompany = contractTemplateService.lambdaQuery().eq(ContractTemplate::getVisible, 1).eq(ContractTemplate::getTemplateCode, "powerOfAttorneyCompany_A").one();
            ContractTemplate powerOfAttorneyPerson = contractTemplateService.lambdaQuery().eq(ContractTemplate::getVisible, 1).eq(ContractTemplate::getTemplateCode, "powerOfAttorneyPerson_A").one();
            ContractTemplate powerOfAttorneyPersonOne = contractTemplateService.lambdaQuery().eq(ContractTemplate::getVisible, 1).eq(ContractTemplate::getTemplateCode, "powerOfAttorneyPerson_B").one();
            if (powerOfAttorneyCompany == null || powerOfAttorneyPerson == null || powerOfAttorneyPersonOne == null) {
                throw new BusinessException(CodeUtil.FAIL, "无模版，请联系开发检查模版！");
            }
            //公司模版
            AttachmentDto attachmentDto = AttachmentDto.builder()
                    .setEntityId(powerOfAttorneyCompany.getId())
                    .setChildEnum(FileModuleDict.ChildEnum.CONTRACT_TEMPLATE_FILE)
                    .build();
            List<FileStorage> files = sysAttachmentRefService.getFiles(attachmentDto);
            FileStorage fileStorage = files.get(0);
            File fileCompany = creatCompanyFile(fileStorage, enterprise);
            FinancingFromInvestorsEletronicContract company = uploadToFDD(fileCompany, enterprise.getCorporateName() + "征信查询授权书", enterprise.getFddCustomerId(), "qz", enterprise, 0L);
            list.add(company);
            //个人模版 征信查询授权书
            AttachmentDto attachmentDtoPerson = AttachmentDto.builder()
                    .setEntityId(powerOfAttorneyPerson.getId())
                    .setChildEnum(FileModuleDict.ChildEnum.CONTRACT_TEMPLATE_FILE)
                    .build();
            List<FileStorage> personFiles = sysAttachmentRefService.getFiles(attachmentDtoPerson);
            FileStorage fileStoragePerson = personFiles.get(0);
            File filePerson = creatPersonFile(fileStoragePerson, signaturePersonal, "个人征信查询授权书" + signaturePersonal.getName());
            //个人模版 综合信息查询授权书
            AttachmentDto attachmentDtoPersonOne = AttachmentDto.builder()
                    .setEntityId(powerOfAttorneyPersonOne.getId())
                    .setChildEnum(FileModuleDict.ChildEnum.CONTRACT_TEMPLATE_FILE)
                    .build();
            List<FileStorage> personFilesOne = sysAttachmentRefService.getFiles(attachmentDtoPersonOne);
            FileStorage fileStoragePersonOne = personFilesOne.get(0);
            File filePersonOne = creatPersonFile(fileStoragePersonOne, signaturePersonal, "综合信息查询授权书" + signaturePersonal.getName());
            FinancingFromInvestorsEletronicContract person = uploadToFDD(filePerson, signaturePersonal.getName() + "征信查询授权书", signaturePersonal.getFddCustomerId(), "qz", enterprise, 1L);
            list.add(person);
            FinancingFromInvestorsEletronicContract personOne = uploadToFDD(filePersonOne, signaturePersonal.getName() + "综合信息查询授权书", signaturePersonal.getFddCustomerId(), "qz", enterprise, 2L);
            list.add(personOne);
        }
        //刷新认证状态
        for (FinancingFromInvestorsEletronicContract financingFromInvestorsEletronicContract : list) {
            boolean auth;
            String authUrl;
            String faddCustomerId;
            String customerType;
            ElectronicSignaturePersonal electronicSignaturePersonal = electronicSignaturePersonalService.lambdaQuery().eq(ElectronicSignaturePersonal::getFddCustomerId, financingFromInvestorsEletronicContract.getFddCustomerId()).one();
            if (electronicSignaturePersonal != null) {
                customerType = "1";
                faddCustomerId = electronicSignaturePersonal.getFddCustomerId();
                auth = electronicSignaturePersonal.getAuthStatus() == 0 ? false : true;
                authUrl = electronicSignaturePersonal.getShortAuthUrl();
            } else {
                customerType = "2";
                faddCustomerId = enterprise.getFddCustomerId();
                auth = enterprise.getAuthStatus();
                authUrl = enterprise.getShortAuthUrl();
            }
            if (!auth) {
                String authInfo = ElectronicSignatureUtil.getAuthInfo(faddCustomerId, customerType);
                if ("1".equals(authInfo)) {
                    auth = true;
                    if (electronicSignaturePersonal != null) {
                        electronicSignaturePersonal.setAuthStatus(1);
                        electronicSignaturePersonalService.updateById(electronicSignaturePersonal);
                    } else {
                        enterprise.setAuthStatus(true);
                        enterpriseService.updateById(enterprise);
                    }
                }
            }
            financingFromInvestorsEletronicContract.setAuthStatus(auth);
            financingFromInvestorsEletronicContract.setShortAuthUrl(authUrl);
            if (auth) {
                //刷新签署状态
                if (!financingFromInvestorsEletronicContract.getSignStatus()) {
                    String signResult = ElectronicSignatureUtil.getSignInfo(financingFromInvestorsEletronicContract.getFddCustomerId(), financingFromInvestorsEletronicContract.getContractId(), financingFromInvestorsEletronicContract.getTransactionId());
                    if (signResult.equals("3000")) {
                        financingFromInvestorsEletronicContractService.doSignCallBack(financingFromInvestorsEletronicContract);
                    }
                }
                //填充人脸识别照片
                if (financingFromInvestorsEletronicContract.getFaceFileId() == null) {
                    Long faceId = null;
                    String type;
                    if (electronicSignaturePersonal == null) {
                        List<FileStorageDO> enterpriserFace = fileService.findByBusinessId(enterprise.getId(), FileModuleDict.ChildEnum.ENTERPRISE_FACE_ID.getCode());
                        if (CollectionUtil.isNotEmpty(enterpriserFace)) {
                            FileStorageDO fileStorageDO = enterpriserFace.get(0);
                            faceId = fileStorageDO.getId();
                        }
                        financingFromInvestorsEletronicContract.setAuthStatus(enterprise.getAuthStatus());
                        financingFromInvestorsEletronicContract.setAuthUrl(enterprise.getAuthUrl());
                        financingFromInvestorsEletronicContract.setShortAuthUrl(enterprise.getShortAuthUrl());
                        type = "2";
                    } else {
                        List<FileStorageDO> enterpriserFace = fileService.findByBusinessId(electronicSignaturePersonal.getId(), FileModuleDict.ChildEnum.PERSON_FACE_ID.getCode());
                        if (CollectionUtil.isNotEmpty(enterpriserFace)) {
                            FileStorageDO fileStorageDO = enterpriserFace.get(0);
                            faceId = fileStorageDO.getId();
                        }
                        financingFromInvestorsEletronicContract.setAuthStatus(electronicSignaturePersonal.getAuthStatus() == 0 ? false : true);
                        financingFromInvestorsEletronicContract.setAuthUrl(electronicSignaturePersonal.getAuthUrl());
                        financingFromInvestorsEletronicContract.setShortAuthUrl(electronicSignaturePersonal.getShortAuthUrl());
                        type = "1";
                    }
                    if (faceId == null) {
                        String result = ElectronicSignatureUtil.getAuthInfoDetail(financingFromInvestorsEletronicContract.getFddCustomerId(), type);
                        if (StringUtil.isNotEmpty(result)) {
                            JSONObject jsonObject = JSONObject.parseObject(result);
                            if (jsonObject != null) {
                                JSONObject data = jsonObject.getJSONObject("data");
                                String back = null;
                                if (data != null) {
                                    if (type.equals("1")) {
                                        JSONObject person = data.getJSONObject("person");
                                        String photoUuid = person.getString("photoUuid");
                                        back = ElectronicSignatureUtil.downloadFaceFile(photoUuid);
                                    } else {
                                        JSONObject manager = data.getJSONObject("manager");
                                        String photoUuid = manager.getString("photoUuid");
                                        back = ElectronicSignatureUtil.downloadFaceFile(photoUuid);
                                    }
                                }
                                if(StringUtil.isNotEmpty(back)){
                                    JSONObject object = JSONObject.parseObject(back);
                                    String value = object.getString("data");

                                    FileStorage backFile = fileStorageService.addFileStorageWithBelongAndName(financingFromInvestorsEletronicContract.getContractId(),
                                            FileModuleDict.ChildEnum.FDD_FACE_FILE, value, null);
                                    fileStorageService.saveAttachmentRefByFileStorage(backFile,
                                            FileModuleDict.ChildEnum.FDD_FACE_FILE,
                                            financingFromInvestorsEletronicContract.getId());
                                    financingFromInvestorsEletronicContract.setFaceFileId(backFile.getId());
                                    financingFromInvestorsEletronicContractService.updateById(financingFromInvestorsEletronicContract);
                                }
                            }
                        }
                    }
                }
            }
            if(financingFromInvestorsEletronicContract.getFaceFileId() != null){
                FileStorage fileStorage = fileStorageService.get(financingFromInvestorsEletronicContract.getFaceFileId());
                financingFromInvestorsEletronicContract.setFaceFile(fileStorage);
            }
        }
        return list;
    }

    @Override
    public GridPage<FinancingFromInvestors> findBySearch(FinancingFromInvestorsSearchVo financingFromInvestorsSearchVo) {
        Long currentUserId = ThreadLocalUtil.getUserId();
        List<UserOrgRel> userOrgRelList = userService.findOrgRelByUserId(currentUserId);
        List<Long> userIdList;
        List<UserRoleRel> userRoleRels = roleService.getUserRoleByUserId(ThreadLocalUtil.getUserId());
        List<Long> pers = permissionService.getPermissionIdByUserId(ThreadLocalUtil.getUserId());
        Permission per = permissionService.getByEname(ConstantConversionUtil.VIEW_ALL_SMB);
        if (per == null) {
            throw new ValidatorException(CodeUtil.FAIL, "数据可见-查看全员权限不能为空");
        }
        if (pers.contains(per.getId())
                || ThreadLocalUtil.getUser().getLoginName().equals(ConstantConversionUtil.ADMINISTORIES)) {
            userIdList = null;
        } else {
            userIdList = new ArrayList<>();
            if (CollectionUtil.isNotEmpty(userRoleRels)) {
                for (UserOrgRel userOrgRel : userOrgRelList) {
                    if (userOrgRel.isUserIdentity()) {
                        // 如果是部门负责人，获取部门下所有人员Id
                        List<Long> userIds = userService.findUserIdByOrgId(userOrgRel.getOrgId());
                        userIdList.addAll(userIds);
                    } else {
                        userIdList.add(currentUserId);
                    }
                }
            }
        }
        financingFromInvestorsSearchVo.setUserIds(userIdList);
        Page page = PageHelper.startPage(financingFromInvestorsSearchVo.getPage(), financingFromInvestorsSearchVo.getRows());
        List<FinancingFromInvestors> list = baseMapper.findBySearch(financingFromInvestorsSearchVo);
        for (FinancingFromInvestors financingFromInvestors : list) {
            SupplierInfo supplierInfo = supplierInfoService.get(financingFromInvestors.getSupplierInfoId());
            Enterprise enterprise = enterpriseService.getById(supplierInfo.getcEnterpriseId());
            financingFromInvestors.setEnterprise(enterprise);
            //判断数据的状态是否支持续授信
            List<FinancingFromInvestorsHistory> financingFromInvestorsHistories = financingFromInvestorsHistoryService.lambdaQuery().eq(FinancingFromInvestorsHistory::getFinancingFromInvestorsId, financingFromInvestors.getId()).orderByDesc(FinancingFromInvestorsHistory::getId).list();
            if(CollectionUtil.isNotEmpty(financingFromInvestorsHistories)){
                //查询最后一次的回调结果
                FinancingFromInvestorsHistory financingFromInvestorsHistory = financingFromInvestorsHistories.get(0);
                FinancingFromInvestorsNotice financingFromInvestorsNotice = financingFromInvestorsNoticeService.lambdaQuery().eq(FinancingFromInvestorsNotice::getFinancingFromInvestorsHistoryId, financingFromInvestorsHistory.getId()).one();
                if (financingFromInvestorsNotice != null && financingFromInvestorsNotice.getExpiringDate().compareTo(new Date()) < 1) {
                    financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_12.getCode());
                    this.updateById(financingFromInvestors);
                }
                Dictionary dictionary = dictionaryService.getByCode(financingFromInvestorsNotice.getLimitStatus());
                financingFromInvestorsNotice.setLimitStatus(dictionary.getName());
                financingFromInvestors.setFinancingFromInvestorsNotice(financingFromInvestorsNotice);
            }
            Dictionary dictionary = dictionaryService.getByCode(financingFromInvestors.getCreditStatus());
            financingFromInvestors.setCreditStatus(dictionary.getName());
        }
        return new GridPage<>(financingFromInvestorsSearchVo.getRows(), financingFromInvestorsSearchVo.getPage(), page.getTotal(), list);
    }

    @Override
    public FinancingFromInvestors getDetail(Long id) {
        FinancingFromInvestors financingFromInvestors = this.getById(id);
        String registeredAddress = financingFromInvestors.getRegisteredAddress();
        String[] split = registeredAddress.split(",");
        List<Long> list = Arrays.stream(split).map(r -> Long.valueOf(r)).collect(Collectors.toList());
        financingFromInvestors.setRegisteredAddressCodeList(list);
        FileStorageDO fileStorageDO = fileService.findOneByBusinessId(id, FileModuleDict.ChildEnum.SMB_BUSINESS_LICENSE_FILE.getCode());
        financingFromInvestors.setFileStorageDO(fileStorageDO);
        //法人信息
        FinancingFromInvestorsPerson financingFromInvestorsPerson = financingFromInvestorsPersonService.lambdaQuery().eq(FinancingFromInvestorsPerson::getFinancingFromInvestorsId, id).one();
        if (financingFromInvestorsPerson != null) {
            FileStorageDO front = fileService.findOneByBusinessId(financingFromInvestorsPerson.getId(), FileModuleDict.ChildEnum.SMB_PERSON_FRONT_FILE.getCode());
            FileStorageDO back = fileService.findOneByBusinessId(financingFromInvestorsPerson.getId(), FileModuleDict.ChildEnum.SMB_PERSON_BACK_FILE.getCode());
            financingFromInvestorsPerson.setFront(front);
            financingFromInvestorsPerson.setBack(back);
        }
        financingFromInvestors.setFinancingFromInvestorsPerson(financingFromInvestorsPerson);
        //合同文件
        List<FinancingFromInvestorsContract> financingFromInvestorsContracts = financingFromInvestorsContractService.lambdaQuery().eq(FinancingFromInvestorsContract::getFinancingFromInvestorsId, id).list();
        if (CollectionUtil.isNotEmpty(financingFromInvestorsContracts)) {
            for (FinancingFromInvestorsContract financingFromInvestorsContract : financingFromInvestorsContracts) {
                FileStorageDO file = fileService.findOneByBusinessId(financingFromInvestorsContract.getId(), FileModuleDict.ChildEnum.SMB_CONTRACT_FILE.getCode());
                financingFromInvestorsContract.setFileStorages(file);
            }
        }
        financingFromInvestors.setFinancingFromInvestorsContracts(financingFromInvestorsContracts);
        //授权书
        SupplierInfo supplierInfo = supplierInfoService.get(financingFromInvestors.getSupplierInfoId());
        //企业信息
        Enterprise enterprise = enterpriseService.getById(supplierInfo.getcEnterpriseId());
        financingFromInvestors.setEnterprise(enterprise);
        List<FinancingFromInvestorsEletronicContract> financingFromInvestorsEletronicContracts = financingFromInvestorsEletronicContractService.lambdaQuery().eq(FinancingFromInvestorsEletronicContract::getcEnterpriseId, enterprise.getId()).list();
        if (CollectionUtil.isNotEmpty(financingFromInvestorsEletronicContracts)) {
            for (FinancingFromInvestorsEletronicContract financingFromInvestorsEletronicContract : financingFromInvestorsEletronicContracts) {
                boolean auth;
                String authUrl;
                String faddCustomerId;
                String customerType;
                ElectronicSignaturePersonal electronicSignaturePersonal = electronicSignaturePersonalService.lambdaQuery().eq(ElectronicSignaturePersonal::getFddCustomerId, financingFromInvestorsEletronicContract.getFddCustomerId()).one();
                if (electronicSignaturePersonal != null) {
                    customerType = "1";
                    faddCustomerId = electronicSignaturePersonal.getFddCustomerId();
                    auth = electronicSignaturePersonal.getAuthStatus() == 0 ? false : true;
                    authUrl = electronicSignaturePersonal.getShortAuthUrl();
                } else {
                    customerType = "2";
                    faddCustomerId = enterprise.getFddCustomerId();
                    auth = enterprise.getAuthStatus();
                    authUrl = enterprise.getShortAuthUrl();
                }
                if (!auth) {
                    String authInfo = ElectronicSignatureUtil.getAuthInfo(faddCustomerId, customerType);
                    if ("1".equals(authInfo)) {
                        auth = true;
                        if (electronicSignaturePersonal != null) {
                            electronicSignaturePersonal.setAuthStatus(1);
                            electronicSignaturePersonalService.updateById(electronicSignaturePersonal);
                        } else {
                            enterprise.setAuthStatus(true);
                            enterpriseService.updateById(enterprise);
                        }
                    }
                }
                financingFromInvestorsEletronicContract.setAuthStatus(auth);
                financingFromInvestorsEletronicContract.setShortAuthUrl(authUrl);
                if (auth) {
                    //刷新签署状态
                    if (!financingFromInvestorsEletronicContract.getSignStatus()) {
                        String signResult = ElectronicSignatureUtil.getSignInfo(financingFromInvestorsEletronicContract.getFddCustomerId(), financingFromInvestorsEletronicContract.getContractId(), financingFromInvestorsEletronicContract.getTransactionId());
                        if (signResult.equals("3000")) {
                            financingFromInvestorsEletronicContractService.doSignCallBack(financingFromInvestorsEletronicContract);
                        }
                    }
                    //填充人脸识别照片
                    if (financingFromInvestorsEletronicContract.getFaceFileId() == null) {
                        Long faceId = null;
                        String type;
                        if (electronicSignaturePersonal == null) {
                            List<FileStorageDO> enterpriserFace = fileService.findByBusinessId(enterprise.getId(), FileModuleDict.ChildEnum.ENTERPRISE_FACE_ID.getCode());
                            if (CollectionUtil.isNotEmpty(enterpriserFace)) {
                                FileStorageDO file = enterpriserFace.get(0);
                                faceId = file.getId();
                            }
                            financingFromInvestorsEletronicContract.setAuthStatus(enterprise.getAuthStatus());
                            financingFromInvestorsEletronicContract.setAuthUrl(enterprise.getAuthUrl());
                            financingFromInvestorsEletronicContract.setShortAuthUrl(enterprise.getShortAuthUrl());
                            type = "2";
                        } else {
                            List<FileStorageDO> enterpriserFace = fileService.findByBusinessId(electronicSignaturePersonal.getId(), FileModuleDict.ChildEnum.PERSON_FACE_ID.getCode());
                            if (CollectionUtil.isNotEmpty(enterpriserFace)) {
                                FileStorageDO file = enterpriserFace.get(0);
                                faceId = file.getId();
                            }
                            financingFromInvestorsEletronicContract.setAuthStatus(electronicSignaturePersonal.getAuthStatus() == 0 ? false : true);
                            financingFromInvestorsEletronicContract.setAuthUrl(electronicSignaturePersonal.getAuthUrl());
                            financingFromInvestorsEletronicContract.setShortAuthUrl(electronicSignaturePersonal.getShortAuthUrl());
                            type = "1";
                        }
                        if (faceId == null) {
                            String result = ElectronicSignatureUtil.getAuthInfoDetail(financingFromInvestorsEletronicContract.getFddCustomerId(), type);
                            if (StringUtil.isNotEmpty(result)) {
                                JSONObject jsonObject = JSONObject.parseObject(result);
                                if (jsonObject != null) {
                                    JSONObject data = jsonObject.getJSONObject("data");
                                    String back = null;
                                    if (data != null) {
                                        if (type.equals("1")) {
                                            JSONObject person = data.getJSONObject("person");
                                            String photoUuid = person.getString("photoUuid");
                                            back = ElectronicSignatureUtil.downloadFaceFile(photoUuid);
                                        } else {
                                            JSONObject manager = data.getJSONObject("manager");
                                            String photoUuid = manager.getString("photoUuid");
                                            back = ElectronicSignatureUtil.downloadFaceFile(photoUuid);
                                        }
                                    }
                                    if(StringUtil.isNotEmpty(back)){
                                        JSONObject object = JSONObject.parseObject(back);
                                        String value = object.getString("data");

                                        FileStorage backFile = fileStorageService.addFileStorageWithBelongAndName(financingFromInvestorsEletronicContract.getContractId(),
                                                FileModuleDict.ChildEnum.FDD_FACE_FILE, value, null);
                                        fileStorageService.saveAttachmentRefByFileStorage(backFile,
                                                FileModuleDict.ChildEnum.FDD_FACE_FILE,
                                                financingFromInvestorsEletronicContract.getId());
                                        financingFromInvestorsEletronicContract.setFaceFileId(backFile.getId());
                                        financingFromInvestorsEletronicContractService.updateById(financingFromInvestorsEletronicContract);
                                    }
                                }
                            }
                        }
                    }
                }
                if(financingFromInvestorsEletronicContract.getFaceFileId() != null){
                    FileStorage fileStorage = fileStorageService.get(financingFromInvestorsEletronicContract.getFaceFileId());
                    financingFromInvestorsEletronicContract.setFaceFile(fileStorage);
                }
            }
        }
        financingFromInvestors.setFinancingFromInvestorsEletronicContracts(financingFromInvestorsEletronicContracts);
        return financingFromInvestors;
    }

    @Override
    public FinancingFromInvestorsNotice getCreditDetail(Long id) {
        List<FinancingFromInvestorsHistory> financingFromInvestorsHistories = financingFromInvestorsHistoryService.lambdaQuery().eq(FinancingFromInvestorsHistory::getFinancingFromInvestorsId, id).orderByDesc(FinancingFromInvestorsHistory::getId).list();
        FinancingFromInvestorsNotice financingFromInvestorsNotice = null;
        if (CollectionUtil.isNotEmpty(financingFromInvestorsHistories)) {
            FinancingFromInvestorsHistory financingFromInvestorsHistory = financingFromInvestorsHistories.get(0);
            financingFromInvestorsNotice = financingFromInvestorsNoticeService.lambdaQuery().eq(FinancingFromInvestorsNotice::getFinancingFromInvestorsHistoryId, financingFromInvestorsHistory.getId()).one();
        }
        return financingFromInvestorsNotice;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public FinancingFromInvestorsNotice findCreditDetail(Long id) {
        FinancingFromInvestors financingFromInvestors = this.getById(id);
        List<FinancingFromInvestorsHistory> financingFromInvestorsHistories =
                financingFromInvestorsHistoryService.lambdaQuery().eq(FinancingFromInvestorsHistory::getFinancingFromInvestorsId, id).orderByDesc(FinancingFromInvestorsHistory::getId).list();
        if(CollectionUtil.isEmpty(financingFromInvestorsHistories)){
            throw new BusinessException(CodeUtil.FAIL,"未查询到授信信息");
        }
        FinancingFromInvestorsHistory financingFromInvestorsHistory = financingFromInvestorsHistories.get(0);
        CreditProcessQueryResp creditProcessQueryResp = smbApiService.creditProcessQuery(financingFromInvestors.getOpenId(), financingFromInvestorsHistory.getBizNo());
        //删除之前的回调
        financingFromInvestorsNoticeService.lambdaUpdate().eq(FinancingFromInvestorsNotice::getFinancingFromInvestorsHistoryId, financingFromInvestorsHistory.getId()).remove();
        //保存这次的回调信息
        FinancingFromInvestorsNotice financingFromInvestorsNotice = new FinancingFromInvestorsNotice();
        financingFromInvestorsNotice.setFinancingFromInvestorsHistoryId(financingFromInvestorsHistory.getId());
        financingFromInvestorsNotice.setApplyDate(DateUtil.parseDate(creditProcessQueryResp.getApplyDate()));
        // 状态 01 待审批 02 审批中 03 审批通过 04审批拒绝 06 已归档 07 处理中 11 运营岗审核
        if (creditProcessQueryResp.getApplyStatus().equals("01")) {
            financingFromInvestorsNotice.setApplyStatus(SmbCreditStatusEnum.STATUS_01.getCode());
            financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_01.getCode());
        } else if (creditProcessQueryResp.getApplyStatus().equals("02")) {
            financingFromInvestorsNotice.setApplyStatus(SmbCreditStatusEnum.STATUS_02.getCode());
            financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_02.getCode());
        } else if (creditProcessQueryResp.getApplyStatus().equals("03")) {
            financingFromInvestorsNotice.setApplyStatus(SmbCreditStatusEnum.STATUS_03.getCode());
            financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_03.getCode());
        } else if (creditProcessQueryResp.getApplyStatus().equals("04")) {
            financingFromInvestorsNotice.setApplyStatus(SmbCreditStatusEnum.STATUS_04.getCode());
            financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_04.getCode());
        } else if (creditProcessQueryResp.getApplyStatus().equals("06")) {
            financingFromInvestorsNotice.setApplyStatus(SmbCreditStatusEnum.STATUS_06.getCode());
            financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_06.getCode());
        } else if (creditProcessQueryResp.getApplyStatus().equals("07")) {
            financingFromInvestorsNotice.setApplyStatus(SmbCreditStatusEnum.STATUS_07.getCode());
            financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_07.getCode());
        } else if (creditProcessQueryResp.getApplyStatus().equals("11")) {
            financingFromInvestorsNotice.setApplyStatus(SmbCreditStatusEnum.STATUS_11.getCode());
            financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_11.getCode());
        }
        this.updateById(financingFromInvestors);
        if(StringUtil.isNotEmpty(creditProcessQueryResp.getNextApplyTime())){
            financingFromInvestorsNotice.setNextApplyDate(DateUtil.parseDate(creditProcessQueryResp.getNextApplyTime()));
        }
        financingFromInvestorsNotice.setRejectReason(creditProcessQueryResp.getReasonMsg());
        // 额度状态 001-未激活；002-生效；004-冻结；005-失效
        if (creditProcessQueryResp.getLimitStatus().equals("001")) {
            financingFromInvestorsNotice.setLimitStatus(SmbLimitStatusEnum.LIMIT_STATUS_001.getCode());
        } else if (creditProcessQueryResp.getLimitStatus().equals("002")) {
            financingFromInvestorsNotice.setLimitStatus(SmbLimitStatusEnum.LIMIT_STATUS_002.getCode());
        } else if (creditProcessQueryResp.getLimitStatus().equals("004")) {
            financingFromInvestorsNotice.setLimitStatus(SmbLimitStatusEnum.LIMIT_STATUS_004.getCode());
        } else if (creditProcessQueryResp.getLimitStatus().equals("005")) {
            financingFromInvestorsNotice.setLimitStatus(SmbLimitStatusEnum.LIMIT_STATUS_005.getCode());
        }
        financingFromInvestorsNotice.setValidityPeriod(DateUtil.parseDate(creditProcessQueryResp.getAuthEndDate()));
        if(creditProcessQueryResp.getCycleType().equals("0")){
            financingFromInvestorsNotice.setLoopFlag("否");
        }else if(creditProcessQueryResp.getCycleType().equals("1")){
            financingFromInvestorsNotice.setLoopFlag("是");
        }
        financingFromInvestorsNotice.setCreditLimit(new BigDecimal(creditProcessQueryResp.getCreditLimit()));
        financingFromInvestorsNotice.setRemainingAvailableCreditLimit(new BigDecimal(creditProcessQueryResp.getCreditLimit()));
        financingFromInvestorsNotice.setEffectiveDate(DateUtil.parseDate(creditProcessQueryResp.getStartDate()));
        financingFromInvestorsNotice.setExpiringDate(DateUtil.parseDate(creditProcessQueryResp.getEndDate()));
        financingFromInvestorsNotice.setApprovedInterestRate(new BigDecimal(creditProcessQueryResp.getActRate()));
        if (creditProcessQueryResp.getPayoffFlag().equals("Y")) {
            financingFromInvestorsNotice.setPendingLoanReceipt("是");
        } else if (creditProcessQueryResp.getPayoffFlag().equals("N")) {
            financingFromInvestorsNotice.setPendingLoanReceipt("否");
        }
        if(creditProcessQueryResp.getGuaranteeFlag().equals("0")){
            financingFromInvestorsNotice.setGuaranteeMethod("质押");
        }else if(creditProcessQueryResp.getGuaranteeFlag().equals("1")){
            financingFromInvestorsNotice.setGuaranteeMethod("抵押");
        }else if(creditProcessQueryResp.getGuaranteeFlag().equals("3")){
            financingFromInvestorsNotice.setGuaranteeMethod("保证");
        }else if(creditProcessQueryResp.getGuaranteeFlag().equals("4")){
            financingFromInvestorsNotice.setGuaranteeMethod("信用");
        }else if(creditProcessQueryResp.getGuaranteeFlag().equals("5")){
            financingFromInvestorsNotice.setGuaranteeMethod("组合(含保证)担保");
        }else if(creditProcessQueryResp.getGuaranteeFlag().equals("6")){
            financingFromInvestorsNotice.setGuaranteeMethod("组合(不含保证)担保");
        }else if(creditProcessQueryResp.getGuaranteeFlag().equals("7")){
            financingFromInvestorsNotice.setGuaranteeMethod("农户联保");
        }else if(creditProcessQueryResp.getGuaranteeFlag().equals("9")){
            financingFromInvestorsNotice.setGuaranteeMethod("其他");
        }
        financingFromInvestorsNotice.setLimitNumber(creditProcessQueryResp.getProductLimitNo());

        financingFromInvestorsNoticeService.save(financingFromInvestorsNotice);

        return financingFromInvestorsNotice;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateStatus() {
        Date date = new Date();
        List<FinancingFromInvestorsNotice> list = financingFromInvestorsNoticeService.lambdaQuery().lt(FinancingFromInvestorsNotice::getExpiringDate, date).list();
        for (FinancingFromInvestorsNotice financingFromInvestorsNotice : list) {
            FinancingFromInvestorsHistory financingFromInvestorsHistory = financingFromInvestorsHistoryService.lambdaQuery().eq(FinancingFromInvestorsHistory::getId, financingFromInvestorsNotice.getFinancingFromInvestorsHistoryId()).one();
            FinancingFromInvestors financingFromInvestors = this.getById(financingFromInvestorsHistory.getFinancingFromInvestorsId());
            financingFromInvestors.setCreditStatus(SmbCreditStatusEnum.STATUS_12.getCode());
            this.updateById(financingFromInvestors);
        }

        List<FinancingFromInvestors> financingFromInvestorsList = this.lambdaQuery().in(FinancingFromInvestors::getCreditStatus, SmbCreditStatusEnum.STATUS_01.getCode(), SmbCreditStatusEnum.STATUS_02.getCode(), SmbCreditStatusEnum.STATUS_03.getCode(), SmbCreditStatusEnum.STATUS_11.getCode()).list();
        for (FinancingFromInvestors financingFromInvestors : financingFromInvestorsList) {
            findCreditDetail(financingFromInvestors.getId());
        }

    }


    private FileStorage downloadFile(String remoteFilePath){

        FileStorage fileStorage = new FileStorage(null, FileModuleDict.ChildEnum.SMB_SING_FILE, UUID.randomUUID().toString(), ".pdf");

        String absolutePath = fileStorage.getAbsolutePath();
        File file = new File(absolutePath);
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }

        DataOutputStream out = null ;
        InputStream inputStream = null;

        try {
            inputStream = smbApiService.downloadFileStream(".pdf", remoteFilePath);
            //读文件流
            out = new DataOutputStream(new FileOutputStream(fileStorage.getAbsolutePath()));
            byte[] buffer = new byte[2048];
            int count = 0;
            while ((count = inputStream.read(buffer)) > 0) {
                out.write(buffer, 0, count);
            }
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return fileStorageService.add(fileStorage);
    }

    private File creatCompanyFile(FileStorage fileStorage, Enterprise enterprise) {
        try {
            String absolutePath = fileStorage.getAbsolutePath();
            Map<String, Object> map = new HashMap<>();
            map.put("name", enterprise.getCorporateName());
            map.put("socialCode", enterprise.getSocialCreditCode());
            map.put("date", DateUtil.formatDate(new Date()));
            String path = FileUtil.getAbsolutePath("企业综合信息查询授权书" + enterprise.getCorporateName(), ".docx", FileModuleDict.ChildEnum.CONTRACT_FILE_PREVIEW);
            XWPFTemplate template = XWPFTemplate.compile(absolutePath).render(map);
            File file = new File(path);
            if (!file.exists()) {
                FileUtils.createParentDirectories(file);
            }
            template.writeToFile(path);
            return file;
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException(CodeUtil.FAIL, "制作合同失败" + e.getMessage());
        }
    }

    private File creatPersonFile(FileStorage fileStorage, ElectronicSignaturePersonal signaturePersonal, String fileName) {
        try {
            String absolutePath = fileStorage.getAbsolutePath();
            Map<String, Object> map = new HashMap<>();
            map.put("name", signaturePersonal.getName());
            map.put("idCard", signaturePersonal.getSocialCreditCode());
            map.put("date", DateUtil.formatDate(new Date()));
            String path = FileUtil.getAbsolutePath(fileName, ".docx", FileModuleDict.ChildEnum.CONTRACT_FILE_PREVIEW);
            XWPFTemplate template = XWPFTemplate.compile(absolutePath).render(map);
            File file = new File(path);
            if (!file.exists()) {
                FileUtils.createParentDirectories(file);
            }
            template.writeToFile(path);
            return file;
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException(CodeUtil.FAIL, "制作合同失败" + e.getMessage());
        }
    }

    private FinancingFromInvestorsEletronicContract uploadToFDD(File file, String docTitle, String customerId, String keyword, Enterprise enterprise, Long contractType) {

        String suffix = FileUtil.getSuffix(file.getName());
        String newPath = file.getAbsolutePath().replace(suffix, ".pdf");
        File pdfFile = OfficeConverterUtil.office2Pdf(file.getAbsolutePath(), newPath);
        //上传授权书
        String contractId = UUID.randomUUID().toString().trim().replaceAll("-", "").toUpperCase();
        ElectronicSignatureUtil.uploadContract(contractId, docTitle, pdfFile);
        //授权书的交易号
        String certificateTransactionId = UUID.randomUUID().toString().trim().replaceAll("-", "").toUpperCase();
        //批量手动签
        List<FddSignData> fddSignDataList = new ArrayList<FddSignData>();
        FddSignData fddSignData = new FddSignData();
        fddSignData.setContractId(contractId);
        fddSignData.setContractType("authorizedEnterprise");
        fddSignData.setSignKeyword(keyword);
        fddSignData.setTransactionId(certificateTransactionId);
        fddSignDataList.add(fddSignData);
        //签署长连接
        String certificateContractLink = ElectronicSignatureUtil.manualBatchSign(customerId, fddSignDataList, docTitle);
        //签署短链接
        String certificateShortUrl = ElectronicSignatureUtil.shortUrl(null, certificateContractLink);
        FinancingFromInvestorsEletronicContract financingFromInvestorsEletronicContract = new FinancingFromInvestorsEletronicContract();
        financingFromInvestorsEletronicContract.setcEnterpriseId(enterprise.getId());
        financingFromInvestorsEletronicContract.setContractType(contractType);
        financingFromInvestorsEletronicContract.setFddCustomerId(customerId);
        financingFromInvestorsEletronicContract.setContractName(docTitle);
        financingFromInvestorsEletronicContract.setTransactionId(certificateTransactionId);
        financingFromInvestorsEletronicContract.setContractId(contractId);
        financingFromInvestorsEletronicContract.setContractLink(certificateContractLink);
        financingFromInvestorsEletronicContract.setKeyword(keyword);
        financingFromInvestorsEletronicContract.setShortContractLink(certificateShortUrl);
        financingFromInvestorsEletronicContract.setShortContractLinkCreate(new Date());
        financingFromInvestorsEletronicContract.setSignStatus(false);
        financingFromInvestorsEletronicContractService.save(financingFromInvestorsEletronicContract);
        return financingFromInvestorsEletronicContract;
    }
}
