package com.sunxung.factoring.service.smb.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.sunxung.factoring.component.util.ConstantUtil;
import com.sunxung.factoring.entity.GridPage;
import com.sunxung.factoring.entity.entprise.Enterprise;
import com.sunxung.factoring.entity.financing.apply.CFinancingBasicInfo;
import com.sunxung.factoring.entity.smb.FinancingLoanInfo;
import com.sunxung.factoring.entity.smb.vo.FinancingLoanInfoSearch;
import com.sunxung.factoring.mapper.smb.FinancingLoanInfoMapper;
import com.sunxung.factoring.service.entprise.IEnterpriseService;
import com.sunxung.factoring.service.financing.apply.ICFinancingBasicInfoService;
import com.sunxung.factoring.service.financing.payment.IPaymentApplyService;
import com.sunxung.factoring.service.smb.IFinancingLoanInfoService;
import com.sunxung.factoring.service.smbApi.api.SMBApiService;
import com.sunxung.factoring.service.smbApi.resp.LoanApplyStatusResp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 资方放款信息 服务实现类
 * </p>
 *
 * @author liutao
 * @since 2024-09-05
 */
@Service
public class FinancingLoanInfoServiceImpl extends ServiceImpl<FinancingLoanInfoMapper, FinancingLoanInfo> implements IFinancingLoanInfoService {

    @Autowired
    private IEnterpriseService enterpriseService;
    @Autowired
    private SMBApiService smbApiService;
    @Autowired
    private ICFinancingBasicInfoService financingBasicInfoService;
    @Autowired
    private IPaymentApplyService paymentApplyService;

    @Override
    public GridPage<FinancingLoanInfo> findBySearch(FinancingLoanInfoSearch search) {

        Page<FinancingLoanInfo> page = PageHelper.startPage(search.getPage(), search.getRows());
        List<FinancingLoanInfo> list = baseMapper.findBySearch(search);
        return new GridPage<>(search.getRows(), search.getPage(), page.getTotal(), list);
    }

    @Override
    public FinancingLoanInfo getDetail(Long id) {
        FinancingLoanInfo financingLoanInfo = getById(id);
        if (financingLoanInfo != null && financingLoanInfo.getcEnterpriseId() != null) {
            Enterprise enterprise = enterpriseService.getById(financingLoanInfo.getcEnterpriseId());
            if (enterprise != null) {
                financingLoanInfo.setSupplierName(enterprise.getCorporateName());
                financingLoanInfo.setSocialCreditCode(enterprise.getSocialCreditCode());
            }
        }
        return financingLoanInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void withdrawStatusQuery() {
        //查询所有放款状态为处理中的放款信息
        List<FinancingLoanInfo> loanInfoList = list(new QueryWrapper<FinancingLoanInfo>().eq("payout_status", "01"));
        if (CollectionUtil.isNotEmpty(loanInfoList)) {
            for (FinancingLoanInfo loanInfo : loanInfoList) {
                LoanApplyStatusResp loanApplyStatusResp = smbApiService.loanApplyStatusQuery(loanInfo.getOpenId(), loanInfo.getPayoutNo());
                if (loanApplyStatusResp != null) {
                    //借据编号
                    loanInfo.setDueBillNo(loanApplyStatusResp.getDuebillNo());
                    //到期日期
                    loanInfo.setMaturityDate(loanApplyStatusResp.getMaturity());
                    //提款申请日期
                    loanInfo.setLoanApplyDate(loanApplyStatusResp.getLoanApplyTime());
                    //放款日期
                    loanInfo.setPayoutDate(loanApplyStatusResp.getPayoutDate());
                    //放款状态
                    loanInfo.setPayoutStatus(loanApplyStatusResp.getStatus());
                    //放款金额
                    loanInfo.setLoanAmount(loanApplyStatusResp.getBusiAmt());
                    if ("03".equals(loanApplyStatusResp.getStatus())) {
                        //如果提款失败，添加失败原因
                        loanInfo.setFailMsg(loanApplyStatusResp.getRemark());
                    }
                    updateById(loanInfo);
                    if ("02".equals(loanApplyStatusResp.getStatus())) {
                        //如果提款成功后、需要向sop推送融资信息
                        if (loanInfo.getcFinancingBasicInfoId() != null) {
                            CFinancingBasicInfo financingBasicInfo = financingBasicInfoService.getById(loanInfo.getcFinancingBasicInfoId());
                            if (financingBasicInfo != null) {
                                paymentApplyService.financingToSop(financingBasicInfo, ConstantUtil.JIANGSU_SUSHANG_BANK, loanInfo.getPayoutDate());
                            }
                        }
                    }
                }
            }
        }

    }
}
