package com.sunxung.factoring.service.smb.impl;

import com.snb.fsos.bean.SMBFileUpload;
import com.snb.fsos.bean.SnbContractSign;
import com.sunxung.factoring.component.exception.BusinessException;
import com.sunxung.factoring.component.util.CodeUtil;
import com.sunxung.factoring.component.util.CollectionUtil;
import com.sunxung.factoring.entity.financing.contract.ContractElectronicCustomerInfo;
import com.sunxung.factoring.entity.financing.contract.ContractElectronicInfo;
import com.sunxung.factoring.entity.smb.*;
import com.sunxung.factoring.entity.sys.FileStorage;
import com.sunxung.factoring.entity.sys.FileStorageDO;
import com.sunxung.factoring.mapper.smb.SmbContractInfoMapper;
import com.sunxung.factoring.service.financing.contract.IContractElectronicCustomerInfoService;
import com.sunxung.factoring.service.financing.contract.IContractElectronicInfoService;
import com.sunxung.factoring.service.smb.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sunxung.factoring.service.smbApi.api.SMBApiService;
import com.sunxung.factoring.service.smbApi.dto.ContractSignBizJsonDto;
import com.sunxung.factoring.service.smbApi.resp.ContractSignResp;
import com.sunxung.factoring.service.sys.FileService;
import com.sunxung.factoring.service.sys.FileStorageService;
import org.apache.xpath.operations.Bool;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;

/**
 * <p>
 * 苏商电子签合同 服务实现类
 * </p>
 *
 * @author liutao
 * @since 2024-09-05
 */
@Service
public class SmbContractInfoServiceImpl extends ServiceImpl<SmbContractInfoMapper, SmbContractInfo> implements ISmbContractInfoService {

    @Autowired
    private IContractElectronicInfoService contractElectronicInfoService;

    @Autowired
    private IContractElectronicCustomerInfoService contractElectronicCustomerInfoService;

    @Autowired
    private SMBApiService smbApiService;

    @Autowired
    private IFinancingFromInvestorsService financingFromInvestorsService;

    @Autowired
    private IFinancingFromInvestorsNoticeService financingFromInvestorsNoticeService;

    @Autowired
    private IFinancingFromInvestorsHistoryService financingFromInvestorsHistoryService;

    @Autowired
    private IFinancingFromInvestorsPersonService financingFromInvestorsPersonService;

    @Autowired
    private FileService fileService;

    @Autowired
    private ISmbContractInfoService smbContractInfoService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public SmbContractInfo sendSmbContract(Long smbContractInfoId, Long smbguaContractContractInfoId) {
        SmbContractInfo smbMasterContractInfo = this.getById(smbContractInfoId);
        List<FinancingFromInvestors> financingFromInvestors = financingFromInvestorsService.lambdaQuery().eq(FinancingFromInvestors::getSupplierInfoId,smbMasterContractInfo.getSupplierId()).orderByDesc(FinancingFromInvestors::getId).list();
        FinancingFromInvestors fromInvestors = financingFromInvestors.get(0);
        FinancingFromInvestorsPerson financingFromInvestorsPerson = financingFromInvestorsPersonService.lambdaQuery().eq(FinancingFromInvestorsPerson::getFinancingFromInvestorsId, fromInvestors.getId()).one();
        List<FinancingFromInvestorsHistory> financingFromInvestorsHistories = financingFromInvestorsHistoryService.lambdaQuery().eq(FinancingFromInvestorsHistory::getFinancingFromInvestorsId, fromInvestors.getId()).orderByDesc(FinancingFromInvestorsHistory::getId).list();
        FinancingFromInvestorsHistory financingFromInvestorsHistory = financingFromInvestorsHistories.get(0);
        FinancingFromInvestorsNotice financingFromInvestorsNotice = financingFromInvestorsNoticeService.lambdaQuery().eq(FinancingFromInvestorsNotice::getFinancingFromInvestorsHistoryId, financingFromInvestorsHistory.getId()).one();
        //苏商签署合同
        SnbContractSign contractSign = new SnbContractSign();
        String bizNo = UUID.randomUUID().toString().replace("-","");
        if(bizNo.length() > 32){
            bizNo = bizNo.substring(0,32);
        }
        contractSign.setBizNo(bizNo);
        contractSign.setProductLimitNo(financingFromInvestorsNotice.getLimitNumber());
        ContractSignBizJsonDto contractSignBizJson = new ContractSignBizJsonDto();
        //主合同
        ContractElectronicInfo contractElectronicInfoMaster = contractElectronicInfoService.getById(smbMasterContractInfo.getContractElectronicInfoId());
        List<ContractElectronicCustomerInfo> contractElectronicCustomerInfosMaster = contractElectronicCustomerInfoService.lambdaQuery().eq(ContractElectronicCustomerInfo::getContractElectronicInfoId, contractElectronicInfoMaster.getId()).list();
        ContractElectronicCustomerInfo contractElectronicCustomerInfoMaster = contractElectronicCustomerInfosMaster.get(0);
        FileStorageDO fileStorageDO = fileService.getById(contractElectronicCustomerInfoMaster.getFileId());
        fileStorageDO.setBusinessId(contractElectronicCustomerInfoMaster.getId());
        fileService.updateById(fileStorageDO);
        List<SMBFileUpload> fileList = new ArrayList<>();
        SMBFileUpload smbFileUpload = new SMBFileUpload();
        smbFileUpload.setFilePath(fileStorageDO.getAbsolutePath());
        smbFileUpload.setFileType("924");
        smbFileUpload.setFileName(fileStorageDO.getOriginalName());
        fileList.add(smbFileUpload);
        //担保合同
        SmbContractInfo smbGuaContractContractInfo = this.getById(smbguaContractContractInfoId);
        ContractElectronicInfo contractElectronicInfoGuaContract = contractElectronicInfoService.getById(smbGuaContractContractInfo.getContractElectronicInfoId());
        List<ContractElectronicCustomerInfo> contractElectronicCustomerInfosGuaContract = contractElectronicCustomerInfoService.lambdaQuery().eq(ContractElectronicCustomerInfo::getContractElectronicInfoId, contractElectronicInfoGuaContract.getId()).list();
        ContractElectronicCustomerInfo contractElectronicCustomerInfoGuaContract = contractElectronicCustomerInfosGuaContract.get(0);
        FileStorageDO storageDO = fileService.getById(contractElectronicCustomerInfoGuaContract.getFileId());
        storageDO.setBusinessId(contractElectronicCustomerInfoGuaContract.getId());
        fileService.updateById(storageDO);
        SMBFileUpload smbFileUploadGua = new SMBFileUpload();
        smbFileUploadGua.setFilePath(storageDO.getAbsolutePath());
        smbFileUploadGua.setFileType("924");
        smbFileUploadGua.setFileName(storageDO.getOriginalName());
        fileList.add(smbFileUploadGua);
        String smbMasterFileId;
        String smbGuaFileId;
        try {
            Map<String, String> map = smbApiService.fileStreamUpload(fileList);
            if(map.size() == 0){
                throw new BusinessException(CodeUtil.FAIL,"文件上传失败！");
            }
            Set<String> keySet = map.keySet();
            for (String key : keySet) {
                if(key.equals(fileStorageDO.getOriginalName())){
                    smbMasterFileId = map.get(key);
                    contractSignBizJson.setType310SdossId(smbMasterFileId);
                }else if(key.equals(storageDO.getOriginalName())){
                    smbGuaFileId = map.get(key);
                    contractSignBizJson.setType104SdossId(smbGuaFileId);

                }else {
                    throw new BusinessException(CodeUtil.FAIL,"文件上传失败！");
                }
            }
        } catch (Exception e) {
            throw new BusinessException(CodeUtil.FAIL,"文件上传失败！");
        }
        contractSignBizJson.setContractNo(smbMasterContractInfo.getContractNumber());
        contractSignBizJson.setGuaContractNo(smbGuaContractContractInfo.getContractNumber());
        contractSignBizJson.setGuaranteeAmt(smbGuaContractContractInfo.getAmount());
        contractSignBizJson.setGuaranteeName(financingFromInvestorsPerson.getName());
        contractSignBizJson.setGuaranteeCertNo(financingFromInvestorsPerson.getSocialCreditCode());
        ContractSignResp contractSignResp = smbApiService.contractSign(contractSign, contractSignBizJson, fromInvestors.getOpenId());
        Boolean signStatus;
        if(contractSignResp.getStatus().equals("01")){
            signStatus = true;
        }else if(contractSignResp.getStatus().equals("02")){
            signStatus = false;
        }else {
            throw new BusinessException(CodeUtil.FAIL,"合同签署失败！");
        }
        smbMasterContractInfo.setFailReason(contractSignResp.getRemark());
        smbMasterContractInfo.setSignStatus(signStatus);
        this.updateById(smbMasterContractInfo);
        smbGuaContractContractInfo.setSignStatus(signStatus);
        smbGuaContractContractInfo.setFailReason(contractSignResp.getRemark());
        this.updateById(smbGuaContractContractInfo);
        return smbGuaContractContractInfo;
    }
}
