package com.sunxung.factoring.service.smbApi.api;

import com.snb.fsos.bean.*;
import com.sunxung.factoring.service.smbApi.dto.ContractSignBizJsonDto;
import com.sunxung.factoring.service.smbApi.dto.CreditApplyRiskMsgDto;
import com.sunxung.factoring.service.smbApi.dto.LoanCreditRiskMsgDto;
import com.sunxung.factoring.service.smbApi.resp.LoanApplyResp;
import com.sunxung.factoring.service.smbApi.resp.*;

import java.io.InputStream;
import java.util.List;
import java.util.Map;

public interface SMBApiService {

    /**
     * 苏商银行SDK环境加载
     */
    void loadEnv();

    /**
     * 贷款申请
     * 
     * @param loanApplyReq 贷款申请参数
     * @param loanCreditRiskMsg 提款json信息
     * @param openId 用户唯一标识 授信返回的OpenId
     * @return 提款申请响应
     */
    LoanApplyResp loanApply(LoanApply loanApplyReq, LoanCreditRiskMsgDto loanCreditRiskMsg, String openId);

    /**
     * 授信申请
     * 
     * @param creditApply 授信申请参数
     * @return 授信申请响应
     */
    CreditApplyResp creditApply(ProductCreditApplyV2 creditApply, CreditApplyRiskMsgDto creditApplyRiskMsg);

    /**
     * 授信状态查询
     * 
     * @param openId 用户唯一标识 授信返回的OpenId
     * @param bizNo 业务流水号
     * @return 授信状态查询响应
     */
    CreditProcessQueryResp creditProcessQuery(String openId, String bizNo);

    /**
     * 合同签署
     * 
     * @param contractSign 合同签署参数
     * @return 合同签署响应
     */
    ContractSignResp contractSign(SnbContractSign contractSign, ContractSignBizJsonDto contractSignBizJson,
            String openId);

    /**
     * 文件流上传
     * 
     * @param fileList 文件流列表
     * @return 文件流上传响应 Map<文件名, 文件路径>
     */
    Map<String, String> fileStreamUpload(List<SMBFileUpload> fileList) throws Exception;

    /**
     * 单笔文件下载
     * 
     * @param fileType 文件类型
     * @param remoteFilePath 远程文件路径
     * @return 文件流
     */
    InputStream downloadFileStream(String fileType, String remoteFilePath) throws Exception;

    /**
     * 期限查询
     * 
     * @return 期限列表
     */
    List<TermQueryResp> termQuery();

    /***
     * 提款状态查询
     * 
     * @param openId 用户唯一标识 授信返回的OpenId
     * @param bizNo 业务流水号 唯一
     * @return 提款状态查询响应
     */
    LoanApplyStatusResp loanApplyStatusQuery(String openId, String bizNo);

    /***
     * 还款计划试算
     * 
     * @param repayPlanTrialReq 还款计划试算请求参数
     * @param openId 用户唯一标识 授信返回的OpenId
     * @return 还款计划试算响应
     */
    RepayPlanTrialResp repayPlanTrial(RepayPlanTrial repayPlanTrialReq, String openId);

    /**
     * 账户信息查询
     * 
     * @param balanceQuery 账户余额查询请求参数
     * @param openId 用户唯一标识 授信返回的OpenId
     * @return 账户信息查询响应
     */
    BalanceQueryResp balanceQuery(BalanceQuery balanceQuery, String openId);

    /**
     * 还款申请
     * 
     * @param repayApplyReq 还款申请请求参数
     * @param openId 用户唯一标识 授信返回的OpenId
     * @return
     */
    RepayStatusResp repayApply(RepayApply repayApplyReq, String openId);

    /**
     * 还款状态查询
     * 
     * @param openId 用户唯一标识 授信返回的OpenId
     * @param bizNo 业务流水号
     * @param duebillNo 借据号
     * @return 还款状态查询响应
     */
    RepayStatusResp repayStatusQuery(String openId, String bizNo, String duebillNo);

    /**
     * 还款计划查询
     * 
     * @param openId 用户唯一标识 授信返回的OpenId
     * @param duebillNo 借据号
     * @return 还款计划查询响应
     */
    RepayPlanResp repayPlanQuery(String openId, String duebillNo);

    /**
     * 账户信息查询
     * 
     * @param accountInfoQuery 账户信息查询请求参数
     * @param openId 用户唯一标识 授信返回的OpenId
     * @return 账户信息查询响应
     */
    List<AccountInfoQueryResp> accountInfoQuery(AccountInfoQuery accountInfoQuery, String openId);

    /**
     * 还款试算
     * 
     * @param repayTrialReq 还款试算请求参数
     * @param openId 用户唯一标识 授信返回的OpenId
     * @return 还款试算响应
     */
    RepayTrialResp repayTrial(RepayTrial repayTrialReq, String openId);

    /**
     * 借据查询
     * 
     * @param duebillQuery 借据查询请求参数
     * @param openId 用户唯一标识 授信返回的OpenId
     * @return 借据查询响应
     */
    List<DuebillQueryResp> duebillQuery(DuebillQuery duebillQuery, String openId);

    /**
     * Lpr 利率查询
     * 
     * @param intRateTerm 利率期限 输入的实际贷款期限，单位必须是M且必须大于0M，例如3M、13M
     * @return Lpr利率查询响应
     */
    String smbLprRateQuery(String intRateTerm);
}
