package com.sunxung.factoring.service.smbApi.api.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.snb.fsos.bean.*;
import com.snb.fsos.util.*;
import com.sunxung.factoring.component.exception.ValidatorException;
import com.sunxung.factoring.component.util.CodeUtil;
import com.sunxung.factoring.component.util.StringUtil;
import com.sunxung.factoring.entity.smbApi.SMBRequestLogPO;
import com.sunxung.factoring.service.smbApi.api.SMBApiService;
import com.sunxung.factoring.service.smbApi.api.SMBRequestLogService;
import com.sunxung.factoring.service.smbApi.configuration.SMBConfiguration;
import com.sunxung.factoring.service.smbApi.constants.SMBConstants;
import com.sunxung.factoring.service.smbApi.constants.TransCodeEnum;
import com.sunxung.factoring.service.smbApi.dto.*;
import com.sunxung.factoring.service.smbApi.resp.CommonResponse;
import com.sunxung.factoring.service.smbApi.resp.LoanApplyResp;
import com.sunxung.factoring.service.smbApi.resp.*;
import jodd.util.MimeTypes;
import org.apache.commons.io.IOUtils;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import java.util.HashMap;
import java.util.Map;
import java.io.*;
import java.util.*;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

@Component
public class SMBApiServiceImpl implements SMBApiService {

    private static final ValidatorFactory validatorFactory = Validation.buildDefaultValidatorFactory();
    private static final Logger logger = LogManager.getLogger(SMBApiServiceImpl.class);
    @Resource
    private SMBConfiguration smbConfiguration;
    @Resource
    private SMBRequestLogService smbRequestLogService;

    @Override
    public void loadEnv() {
        logger.info("------开始加载SMB系统对接环境");
        // 初始设置一次即可，四个方法顺序不能变动 ↓
        if ("sit".equals(smbConfiguration.getEnv())) {
            SnbSignUtil.setEnv(InitData.ENV.sit);
        } else if ("uat".equals(smbConfiguration.getEnv())) {
            SnbSignUtil.setEnv(InitData.ENV.uat);
        } else if ("prd".equals(smbConfiguration.getEnv())) {
            SnbSignUtil.setEnv(InitData.ENV.prd);
        }
        // 赋值加密解密SSL认证证书目录
        SnbSignUtil.setFILEPATH(smbConfiguration.getCertFilePath());
        SnbSignUtil.setKeyMetadata("chink2021", "chink2021", SnbSignUtil.FILEPATH);
        // 初始化API
        SnbSignUtil.initApi(smbConfiguration.getAppCode(), smbConfiguration.getChannelId());
        logger.info("------SMB系统对接环境加载结束");
    }

    @Override
    public LoanApplyResp loanApply(LoanApply loanApplyReq, LoanCreditRiskMsgDto loanCreditRiskMsg, String openId) {
        loanApplyReq.setMerchantId(smbConfiguration.getMerchantId());
        loanApplyReq.setProductCode(smbConfiguration.getProductCode());
        loanApplyReq.setSysCode(smbConfiguration.getSysCode());
        loanApplyReq.setPlatformCode(smbConfiguration.getPlatformCode());
        validateSubParams(loanCreditRiskMsg);
        loanApplyReq.setCreditRiskMsg(JSONObject.toJSONString(loanCreditRiskMsg));
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(loanApplyReq);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.WITHDRAW_APPLY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), LoanApplyResp.class);
    }

    @Override
    public CreditApplyResp creditApply(ProductCreditApplyV2 creditApply, CreditApplyRiskMsgDto creditApplyRiskMsg) {
        validateSubParams(creditApplyRiskMsg);
        creditApply.setCreditRiskMsg(JSONObject.toJSONString(creditApplyRiskMsg));
        creditApply.setMerchantId(smbConfiguration.getMerchantId());
        creditApply.setPlatformCode(smbConfiguration.getPlatformCode());
        creditApply.setProductCode(smbConfiguration.getProductCode());
        creditApply.setSysCode(smbConfiguration.getSysCode());
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(creditApply);
        snbRequestBean.setTransCode(TransCodeEnum.CREDIT_APPLY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), CreditApplyResp.class);
    }

    private void validateSubParams(SMBSubParam subParam) {
        Validator validator = validatorFactory.getValidator();
        Set<ConstraintViolation<SMBSubParam>> violationSet = validator.validate(subParam);
        StringBuilder resError = new StringBuilder();
        for (ConstraintViolation<SMBSubParam> violation : violationSet) {
            resError.append(violation.getMessage()).append(", ");
        }
        if (violationSet.size() > 0) {
            throw new ValidatorException(CodeUtil.PARAMETER_ERROR, resError.toString());
        }
    }

    @Override
    public CreditProcessQueryResp creditProcessQuery(String openId, String bizNo) {
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        CommonQuery commonQuery = new CommonQuery(smbConfiguration);
        commonQuery.setBizNo(bizNo);
        snbRequestBean.setPayLoadBean(commonQuery);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.CREDIT_STATUS_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), CreditProcessQueryResp.class);
    }

    @Override
    public ContractSignResp contractSign(SnbContractSign contractSign, ContractSignBizJsonDto contractSignBizJson,
                                         String openId) {
        validateSubParams(contractSignBizJson);
        contractSign.setMerchantId(smbConfiguration.getMerchantId());
        contractSign.setPlatformCode(smbConfiguration.getPlatformCode());
        contractSign.setProductCode(smbConfiguration.getProductCode());
        contractSign.setSysCode(smbConfiguration.getSysCode());
        contractSign.setBizJson(JSONObject.toJSONString(contractSignBizJson));
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(contractSign);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.CONTRACT_SIGN.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), ContractSignResp.class);
    }

    @Override
    public Map<String, String> fileStreamUpload(List<SMBFileUpload> fileList) throws Exception {
        String transCode = TransCodeEnum.FILESTREAM_UPLOAD.getCode();
        MultipartEntityBuilder multipartEntityBuilder = MultipartEntityBuilder.create()
                .setMode(HttpMultipartMode.RFC6532);
        Map<String, Object> payload = new HashMap<>();
        payload.put("sceneCode", SMBConstants.SCENE_CODE);
        payload.put("fileDigestMap", getFileDigestMap(fileList, multipartEntityBuilder));
        Map<String, String> head = assembleRequest(transCode, payload);
        Set<String> keys = head.keySet();
        for (String key : keys) {
            multipartEntityBuilder.addPart(key, new StringBody(head.get(key),
                    ContentType.create(MimeTypes.MIME_TEXT_PLAIN, InitData.CHARSET_UTF8)));
        }
        HttpPost httpPost = this.getMultiPost(smbConfiguration.getFileUrl() + smbConfiguration.getAppCode() + "/",
                transCode);
        httpPost.setEntity(multipartEntityBuilder.build());
        HttpResponse response = RequestHelp.httpConnectionManager.getHttpClient().execute(httpPost);
        try (InputStreamReader in = new InputStreamReader(response.getEntity().getContent());
             BufferedReader bfin = new BufferedReader(in)) {
            Map<String, Object> respMap = getRespMap(bfin);
            saveLogAsync(transCode, head, respMap);
            if (null != respMap.get("respCode") && !"00000000".equals(respMap.get("respCode"))) {
                throw new ValidatorException(CodeUtil.FAIL, "文件上传失败" + respMap.get("respMsg"));
            }
            return JSON.parseObject((String) respMap.get("payload"), new TypeReference<Map<String, String>>() {
            });
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    @Override
    public InputStream downloadFileStream(String fileType, String remoteFilePath) throws Exception {
        Map<String, Object> responseMap = downloadFile(fileType, remoteFilePath);
        boolean result = (Boolean) responseMap.get("downloadResult");
        if (!result) {// todo check responseMap 是否有返回失败原因，在这里抛出
            throw new ValidatorException(CodeUtil.FAIL, "文件下载失败");
        }
        return (InputStream) responseMap.get("inputStream");
    }

    @Override
    public List<TermQueryResp> termQuery() {
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        CommonQuery commonQuery = new CommonQuery(smbConfiguration);
        snbRequestBean.setPayLoadBean(commonQuery);
        snbRequestBean.setTransCode(TransCodeEnum.DEADLINE_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseArray(commonResponse.getPayload(), TermQueryResp.class);
    }

    @Override
    public LoanApplyStatusResp loanApplyStatusQuery(String openId, String bizNo) {
        LoanApplyStatusQuery loanApplyStatusQuery = new LoanApplyStatusQuery();
        loanApplyStatusQuery.setMerchantId(smbConfiguration.getMerchantId());
        loanApplyStatusQuery.setProductCode(smbConfiguration.getProductCode());
        loanApplyStatusQuery.setSysCode(smbConfiguration.getSysCode());
        loanApplyStatusQuery.setPlatformCode(smbConfiguration.getPlatformCode());
        loanApplyStatusQuery.setPayoutNo(bizNo);
        //  loanApplyStatusQuery.setBizNo(bizNo);
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(loanApplyStatusQuery);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.WITHDRAW_STATUS_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), LoanApplyStatusResp.class);
    }

    @Override
    public RepayPlanTrialResp repayPlanTrial(RepayPlanTrial repayPlanTrialReq, String openId) {
        repayPlanTrialReq.setMerchantId(smbConfiguration.getMerchantId());
        repayPlanTrialReq.setProductCode(smbConfiguration.getProductCode());
        repayPlanTrialReq.setSysCode(smbConfiguration.getSysCode());
        repayPlanTrialReq.setPlatformCode(smbConfiguration.getPlatformCode());
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(repayPlanTrialReq);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.REPAY_PLAN_TRIAL.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), RepayPlanTrialResp.class);
    }

    @Override
    public BalanceQueryResp balanceQuery(BalanceQuery balanceQuery, String openId) {
        balanceQuery.setMerchantId(smbConfiguration.getMerchantId());
        balanceQuery.setProductCode(smbConfiguration.getProductCode());
        balanceQuery.setSysCode(smbConfiguration.getSysCode());
        balanceQuery.setPlatformCode(smbConfiguration.getPlatformCode());
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(balanceQuery);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.ACCOUNT_BALANCE_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), BalanceQueryResp.class);
    }

    @Override
    public RepayStatusResp repayApply(RepayApply repayApplyReq, String openId) {
        repayApplyReq.setMerchantId(smbConfiguration.getMerchantId());
        repayApplyReq.setProductCode(smbConfiguration.getProductCode());
        repayApplyReq.setSysCode(smbConfiguration.getSysCode());
        repayApplyReq.setPlatformCode(smbConfiguration.getPlatformCode());
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(repayApplyReq);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.REPAY_APPLY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), RepayStatusResp.class);
    }

    @Override
    public RepayStatusResp repayStatusQuery(String openId, String bizNo, String duebillNo) {
        RepayStatusQuery repayStatusQuery = new RepayStatusQuery();
        repayStatusQuery.setMerchantId(smbConfiguration.getMerchantId());
        repayStatusQuery.setProductCode(smbConfiguration.getProductCode());
        repayStatusQuery.setSysCode(smbConfiguration.getSysCode());
        repayStatusQuery.setPlatformCode(smbConfiguration.getPlatformCode());
        repayStatusQuery.setBizNo(bizNo);
        repayStatusQuery.setDuebillNo(duebillNo);
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(repayStatusQuery);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.REPAY_STATUS_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), RepayStatusResp.class);
    }

    @Override
    public RepayPlanResp repayPlanQuery(String openId, String duebillNo) {
        RepayPlanQuery repayPlanQuery = new RepayPlanQuery();
        repayPlanQuery.setMerchantId(smbConfiguration.getMerchantId());
        repayPlanQuery.setProductCode(smbConfiguration.getProductCode());
        repayPlanQuery.setSysCode(smbConfiguration.getSysCode());
        repayPlanQuery.setPlatformCode(smbConfiguration.getPlatformCode());
        repayPlanQuery.setDuebillNo(duebillNo);
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(repayPlanQuery);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.REPAY_PLAN_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), RepayPlanResp.class);
    }

    @Override
    public List<AccountInfoQueryResp> accountInfoQuery(AccountInfoQuery accountInfoQuery, String openId) {
        accountInfoQuery.setMerchantId(smbConfiguration.getMerchantId());
        accountInfoQuery.setProductCode(smbConfiguration.getProductCode());
        accountInfoQuery.setSysCode(smbConfiguration.getSysCode());
        accountInfoQuery.setPlatformCode(smbConfiguration.getPlatformCode());
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(accountInfoQuery);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.ACCOUNT_INFO_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        JSONObject jsonObject = JSON.parseObject(commonResponse.getPayload());
        return JSONObject.parseArray(jsonObject.getJSONArray("accountsList").toString(), AccountInfoQueryResp.class);
    }

    @Override
    public RepayTrialResp repayTrial(RepayTrial repayTrialReq, String openId) {
        repayTrialReq.setMerchantId(smbConfiguration.getMerchantId());
        repayTrialReq.setProductCode(smbConfiguration.getProductCode());
        repayTrialReq.setSysCode(smbConfiguration.getSysCode());
        repayTrialReq.setPlatformCode(smbConfiguration.getPlatformCode());
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(repayTrialReq);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.REPAY_TRIAL.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSONObject.parseObject(commonResponse.getPayload(), RepayTrialResp.class);
    }

    @Override
    public List<DuebillQueryResp> duebillQuery(DuebillQuery duebillQuery, String openId) {
        duebillQuery.setMerchantId(smbConfiguration.getMerchantId());
        duebillQuery.setProductCode(smbConfiguration.getProductCode());
        duebillQuery.setSysCode(smbConfiguration.getSysCode());
        duebillQuery.setPlatformCode(smbConfiguration.getPlatformCode());
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(duebillQuery);
        snbRequestBean.setOpenId(openId);
        snbRequestBean.setTransCode(TransCodeEnum.DUEBILL_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        JSONObject jsonObject = JSON.parseObject(commonResponse.getPayload());
        return JSONObject.parseArray(jsonObject.getJSONArray("duebList").toString(), DuebillQueryResp.class);
    }

    @Override
    public String smbLprRateQuery(String intRateTerm) {
        LprRateQuery lprRateQuery = new LprRateQuery();
        lprRateQuery.setMerchantId(smbConfiguration.getMerchantId());
        lprRateQuery.setProductCode(smbConfiguration.getProductCode());
        lprRateQuery.setSysCode(smbConfiguration.getSysCode());
        lprRateQuery.setPlatformCode(smbConfiguration.getPlatformCode());
        SnbRequestBean snbRequestBean = new SnbRequestBean();
        snbRequestBean.setPayLoadBean(lprRateQuery);
        snbRequestBean.setTransCode(TransCodeEnum.LPR_RATE_QUERY.getCode());
        CommonResponse commonResponse = sendToSmb(snbRequestBean, null);
        if (!SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
            throw new ValidatorException(CodeUtil.FAIL, commonResponse.getRespMsg());
        }
        return JSON.parseObject(commonResponse.getPayload()).getString("intRate");
    }

    /**
     * 发送业务请求到SMB 仅处理业务请求，不处理业务逻辑 业务逻辑请根据业务需求自行处理
     *
     * @param snbRequestBean smb请求参数
     * @param bizMap         业务字段 如用来保存日志
     * @return SMB返回的数据
     */
    private CommonResponse sendToSmb(SnbRequestBean snbRequestBean, Map<String, Object> bizMap) {
        if (null == snbRequestBean) {
            throw new ValidatorException(CodeUtil.PARAMETER_ERROR, "请求参数不能为空");
        }
        if (null == bizMap) {
            bizMap = new HashMap<>();
        }
        // 请求流水号： 商户编号（8位）+交易发起系统交易日期时间（14位）+交易发起方系统流水号（10位）
        snbRequestBean.setChannelSerialNo(SnbSignUtil.getTranSerialNumber(smbConfiguration.getMerchantId(), 10));
        snbRequestBean.setAppCode(smbConfiguration.getAppCode());
        snbRequestBean.setTerminal(SMBConstants.TERMINAL_PC);
        String transCode = snbRequestBean.getTransCode();
        CommonResponse commonResponse = new CommonResponse();
        SMBRequestLogPO smbRequestLogPO = new SMBRequestLogPO();
        smbRequestLogPO.setTransCode(snbRequestBean.getTransCode());
        smbRequestLogPO.setChannelSerialNo(snbRequestBean.getChannelSerialNo());
        smbRequestLogPO.setRequestPayload(JSONObject.toJSONString(snbRequestBean.getPayLoadBean()));
        try {
            Map<String, Object> responseMap = SnbApiHelper.send(snbRequestBean);
            commonResponse.setRespCode(responseMap.getOrDefault("respCode", "").toString());
            commonResponse.setRespMsg(responseMap.getOrDefault("respMsg", "").toString());
            commonResponse.setAlgorithm(
                    responseMap.get("algorithm") == null ? "" : responseMap.get("algorithm").toString());
            commonResponse.setCharset(responseMap.get("charset") == null ? "" : responseMap.get("charset").toString());
            commonResponse.setSerialNo(responseMap.get("serialNo") == null ? "" : responseMap.get("serialNo").toString());
            commonResponse.setChannelSerialNo(responseMap.get("channelSerialNo") == null ? "" : responseMap.get("channelSerialNo").toString());
            commonResponse.setSecretKey(responseMap.get("secretKey") == null ? "" : responseMap.get("secretKey").toString());
            commonResponse.setSignature(responseMap.get("signature") == null ? "" : responseMap.get("signature").toString());
            commonResponse.setPayload(responseMap.get("payload") == null ? "" : responseMap.get("payload").toString());
            smbRequestLogPO.setSmbSerialNo(commonResponse.getSerialNo());
            smbRequestLogPO.setRemark(commonResponse.getRespMsg());
            if (StringUtil.isNotEmpty(commonResponse.getPayload())) {
                smbRequestLogPO.setResponsePayload(commonResponse.getPayload());
            }
            if (SMBConstants.RESP_SUCCESS.equals(commonResponse.getRespCode())) {
                smbRequestLogPO.setSuccess(true);
            } else {
                smbRequestLogPO.setSuccess(false);
            }
        } catch (Exception e) {
            logger.error("请求SMB-{}失败:{}", transCode, e);
            commonResponse.setRespCode(SMBConstants.RESP_FAIL);
            commonResponse.setRespMsg("请求失败，" + e.getMessage());
        }
        CompletableFuture.runAsync(() -> smbRequestLogService.add(smbRequestLogPO)).exceptionally(e -> {
            logger.error("保存请求日志失败:{}", e.getMessage());
            return null;
        });
        return commonResponse;
    }

    /**
     * 计算文件摘要
     *
     * @param fileList               文件列表
     * @param multipartEntityBuilder 请求构造器
     * @return 文件摘要map
     */
    private Map<String, String> getFileDigestMap(List<SMBFileUpload> fileList,
                                                 MultipartEntityBuilder multipartEntityBuilder) {
        Map<String, String> fileDigestMap = new HashMap<>();
        for (int i = 0; i < fileList.size(); i++) {
            SMBFileUpload file = fileList.get(i);
            fileDigestMap.put(SMBConstants.uploadFilePrefix + i + SMBConstants.downloadFileSuffix, file.getFileType());
            byte[] bytes;
            try (InputStream inputStream = new FileInputStream(file.getFilePath())) {
                bytes = IOUtils.toByteArray(inputStream);
                multipartEntityBuilder.addBinaryBody(SMBConstants.uploadFilePrefix + i, new ByteArrayInputStream(bytes),
                        ContentType.MULTIPART_FORM_DATA, file.getFileName());
                fileDigestMap.put(SMBConstants.uploadFilePrefix + i,
                        SnbUtil.generateMD5(new ByteArrayInputStream(bytes)));// 计算摘要
            } catch (IOException e) {
                e.printStackTrace();
                throw new ValidatorException(CodeUtil.FAIL, "文件读取失败");
            }
        }
        return fileDigestMap;
    }

    /**
     * 下载文件
     *
     * @param fileType       文件类型
     * @param remoteFilePath 远程文件路径
     * @return 下载结果
     */
    public Map<String, Object> downloadFile(String fileType, String remoteFilePath) throws Exception {
        String transCode = TransCodeEnum.REMOTEFILE_DOWNLOAD.getCode();
        Map<String, Object> payload = new HashMap<>();
        payload.put("merchantId", smbConfiguration.getMerchantId());
        payload.put("sceneCode", SMBConstants.SCENE_CODE);
        payload.put("remoteFilePath", remoteFilePath);
        payload.put("fileType", fileType);
        Map<String, String> head = assembleRequest(transCode, payload);
        String content = Permutation.sort(head, InitData.CHARSET_UTF8);
        HttpPost httpPost = this.getMultiPost(smbConfiguration.getFileUrl() + smbConfiguration.getAppCode() + "/",
                transCode);
        httpPost.setEntity(new ByteArrayEntity(content.getBytes(InitData.CHARSET_UTF8)));
        HttpResponse response = RequestHelp.httpConnectionManager.getHttpClient().execute(httpPost);
        Header contentTypeHeader = response.getEntity().getContentType();
        if (contentTypeHeader == null
                || !contentTypeHeader.getValue().startsWith(MimeTypes.MIME_APPLICATION_OCTET_STREAM)) {
            try (InputStreamReader in = new InputStreamReader(response.getEntity().getContent(), InitData.CHARSET_UTF8);
                 BufferedReader bfin = new BufferedReader(in)) {
                Map<String, Object> responseMap = getRespMap(bfin);
                saveLogAsync(transCode, head, responseMap);
                responseMap.put("downloadResult", false);
                return responseMap;
            }
        } else {
            saveLogAsync(transCode, head, null);
            Map<String, Object> responseMap = new HashMap<>();
            responseMap.put("downloadResult", true);
            responseMap.put("inputStream", response.getEntity().getContent());
            return responseMap;
        }
    }

    private void saveLogAsync(String transCode, Map<String, String> head, Map<String, Object> responseMap) {
        SMBRequestLogPO smbRequestLogPO = new SMBRequestLogPO();
        smbRequestLogPO.setTransCode(transCode);
        smbRequestLogPO.setChannelSerialNo(head.get("channelSerialNo"));
        if (responseMap == null) {// 文件下载成功
            smbRequestLogPO.setSuccess(true);
            smbRequestLogPO.setRemark("请求成功");
            smbRequestLogPO.setResponsePayload("");
        } else {
            smbRequestLogPO.setRequestPayload(JSONObject.toJSONString(head.get("payload")));
            smbRequestLogPO.setSmbSerialNo((String) responseMap.get("serialNo"));
            smbRequestLogPO.setRemark((String) responseMap.get("serialNo"));
            smbRequestLogPO.setResponsePayload((String) responseMap.get("payload"));
            smbRequestLogPO.setSuccess(SMBConstants.RESP_SUCCESS.equals(responseMap.get("respCode")));
        }
        CompletableFuture.runAsync(() -> smbRequestLogService.add(smbRequestLogPO));
    }

    @SuppressWarnings("rawtypes")
    private Map getRespMap(BufferedReader bufferedReader) throws IOException {
        StringBuilder responseSb = new StringBuilder();
        String line;
        while ((line = bufferedReader.readLine()) != null) {
            responseSb.append(line);
        }
        Map respMap = InitData.OBJECTMAPPER.readValue(responseSb.toString(), Map.class);
        return SnbUtil.extendMap(respMap);
    }

    /**
     * 组装请求报文
     *
     * @param transCode  交易码
     * @param payloadMap 业务数据
     * @return 请求报文
     */
    private Map<String, String> assembleRequest(String transCode, Map<String, Object> payloadMap) throws Exception {
        String rand = SnbUtil.genRandomNum();
        Map<String, String> dataMap = new HashMap<>();
        dataMap.put("appCode", smbConfiguration.getAppCode());// 应用编号
        dataMap.put("channelSerialNo", SnbSignUtil.getTranSerialNumber(smbConfiguration.getMerchantId(), 10));// 流水号
        dataMap.put("timestamp", SnbUtil.getTimestamp());// 时间戳
        dataMap.put("algorithm", InitData.Algorithm);
        dataMap.put("channelId", smbConfiguration.getChannelId());
        dataMap.put("merchantId", smbConfiguration.getMerchantId());
        dataMap.put("secretKey", SnbUtil.encyptRandomNum(rand));// 随机秘钥
        dataMap.put("transCode", transCode);// 交易码
        // 业务数据转成json后使用randomNum加密
        String payloadJson = InitData.OBJECTMAPPER.writeValueAsString(payloadMap);
        String payload = SnbUtil.encyptPayload(payloadJson, rand);
        // 组装签名原文所需的数据
        Map<String, Object> plainMap = new HashMap<>();
        plainMap.putAll(dataMap);
        plainMap.putAll(payloadMap);
        // 生成签名原文plain 并使用商户私钥获签名
        String plain = Permutation.sort(plainMap, InitData.CHARSET_UTF8);
        String signature = SnbUtil.sign(plain, InitData.Algorithm, SnbSignUtil.keyMetadata);
        // 将业务数据加密后的结果和签名值存放到请求报文中
        dataMap.put("payload", payload);
        dataMap.put("signature", signature);
        return dataMap;
    }

    protected HttpPost getMultiPost(String url, String transCode) {
        HttpPost post = new HttpPost(url + transCode);
        post.setHeader("charset", InitData.CHARSET_UTF8);
        post.setHeader("appCode", smbConfiguration.getAppCode());
        post.setHeader("version", "1.0");
        return post;
    }
}
