package com.sunxung.factoring.service.smbApi.file;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Component;

import com.sunxung.factoring.component.exception.ValidatorException;
import com.sunxung.factoring.component.util.CodeUtil;
import com.sunxung.factoring.service.smbApi.configuration.SMBConfiguration;

/**
 * 读取苏商银行SFTP文件工具类
 */
@Component
public class SMBFileLoadUtil {

    private static final String FILE_SEP = "\\|";
    private static final String FILE_SUFFIX = ".txt";
    private static final String OK_SUFFIX = ".ok";
    private static final String DATE_FORMAT = "yyyyMMdd";
    public static final String LOAN_FILE_KEY = "payout";
    public static final String REPAY_FILE_KEY = "repay";
    public static final String COMPENSATION_FILE_KEY = "compen";
    @Resource
    private SMBConfiguration smbConfiguration;
    /*
     * public static List<Map<Integer, Object>> connectSFTPAndLoadFileData(String fileType, LocalDate date) throws
     * JSchException { SftpClient sftpClient = new SftpClient(); sftpClient.connect(); List<Map<Integer, Object>>
     * fileData = loadFileData(fileType, date); sftpClient.disconnect(); return fileData; }
     */

    public List<Map<Integer, Object>> loadFileData(String fileType, LocalDate date) {
        String dateStr = date.format(DateTimeFormatter.ofPattern(DATE_FORMAT));
        String remoteFilePath = smbConfiguration.getSftpPath() + File.separator + dateStr + File.separator;
        File okFile = new File(remoteFilePath + dateStr + OK_SUFFIX);
        if (!okFile.exists()) {
            throw new ValidatorException(CodeUtil.FAIL, "ok file not found");
        }
        File dataFile = new File(remoteFilePath + fileType + dateStr + FILE_SUFFIX);
        if (!dataFile.exists()) {
            throw new ValidatorException(CodeUtil.FAIL, "data file not found");
        }
        List<Map<Integer, Object>> fileData = new ArrayList<>();
        try (BufferedReader reader = new BufferedReader(
                new InputStreamReader(new FileInputStream(dataFile), StandardCharsets.UTF_8))) {
            String line;
            while ((line = reader.readLine()) != null) {
                String[] singleLineData = line.split(FILE_SEP);
                Map<Integer, Object> map = new java.util.HashMap<>();
                for (int i = 0; i < singleLineData.length; i++) {
                    map.put(i, singleLineData[i]);
                }
                fileData.add(map);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return fileData;
    }
}
