package com.sunxung.factoring.service.smbApi.spi;

import com.sunxung.factoring.service.smbApi.dto.CreditNotifyPayloadDto;
import com.sunxung.factoring.service.smbApi.dto.SMBNotifyRespDto;
import com.sunxung.factoring.service.smbApi.dto.WithdrawNotifyPayloadDto;

import java.util.function.Function;

/**
 * SPI注册接口：用于注册苏商银行通知回调处理函数
 */
public interface SMBSpiRegister {

    /**
     * 注册授信结果通知回调处理函数
     *
     * @param creditNotifyHandler 授信结果通知回调处理函数
     */
    void registerCreditNotifyHandler(Function<CreditNotifyPayloadDto, SMBNotifyRespDto> creditNotifyHandler);

    /**
     * 注册提款结果通知回调处理函数
     * 
     * @param withdrawNotifyHandler 提款结果通知回调处理函数
     */
    void registerWithdrawNotifyHandler(Function<WithdrawNotifyPayloadDto, SMBNotifyRespDto> withdrawNotifyHandler);

    /**
     * 根据交易代码查找对应的处理函数
     * 
     * @param transCode
     */
    Function<Object, SMBNotifyRespDto> findHandler(String transCode);
}
