package com.sunxung.factoring.service.smbApi.spi.impl;

import com.sunxung.factoring.service.smbApi.constants.TransCodeEnum;
import com.sunxung.factoring.service.smbApi.dto.CreditNotifyPayloadDto;
import com.sunxung.factoring.service.smbApi.dto.SMBNotifyRespDto;
import com.sunxung.factoring.service.smbApi.dto.WithdrawNotifyPayloadDto;
import com.sunxung.factoring.service.smbApi.spi.SMBSpiRegister;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

@Service
public class SMBSpiRegisterImpl implements SMBSpiRegister {

    private static final Logger logger = LoggerFactory.getLogger(SMBSpiRegisterImpl.class);
    private final Map<String, Function> handlerMap = new HashMap<>();

    @Override
    public void registerCreditNotifyHandler(Function<CreditNotifyPayloadDto, SMBNotifyRespDto> creditNotifyHandler) {
        logger.debug("register credit notify handler");
        handlerMap.put(TransCodeEnum.CREDIT_NOTIFY.getCode(), creditNotifyHandler);
    }

    @Override
    public void registerWithdrawNotifyHandler(
            Function<WithdrawNotifyPayloadDto, SMBNotifyRespDto> withdrawNotifyHandler) {
        logger.debug("register withdraw notify handler");
        handlerMap.put(TransCodeEnum.WITHDRAW_NOTIFY.getCode(), withdrawNotifyHandler);
    }

    @Override
    public Function<Object, SMBNotifyRespDto> findHandler(String transCode) {
        return handlerMap.get(transCode);
    }
}
