package com.sunxung.factoring.service.smbApi.spi.impl;

import com.alibaba.fastjson.JSONObject;
import com.snb.fsos.util.SnbSignUtil;
import com.snb.fsos.util.SnbUtil;
import com.sunxung.factoring.entity.smbApi.SMBRequestLogPO;
import com.sunxung.factoring.service.smbApi.api.SMBRequestLogService;
import com.sunxung.factoring.service.smbApi.configuration.SMBConfiguration;
import com.sunxung.factoring.service.smbApi.constants.TransCodeEnum;
import com.sunxung.factoring.service.smbApi.dto.CreditNotifyPayloadDto;
import com.sunxung.factoring.service.smbApi.dto.SMBNotifyRespDto;
import com.sunxung.factoring.service.smbApi.dto.WithdrawNotifyPayloadDto;
import com.sunxung.factoring.service.smbApi.spi.SMBSpiRegister;
import com.sunxung.factoring.service.smbApi.spi.SMBSpiService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

@Service
public class SMBSpiServiceImpl implements SMBSpiService {

    private static final Logger logger = LoggerFactory.getLogger(SMBSpiServiceImpl.class);
    private static final String SUCCESS_CODE = "00000000";
    @Resource
    private SMBConfiguration smbConfiguration;
    @Resource
    private SMBSpiRegister smbSpiRegister;
    @Resource
    private SMBRequestLogService smbRequestLogService;

    @Override
    public Map handSMBNotifyCallback(Map<String, String[]> reqParameterMap) {
        Map<String, String> parameterMap = new HashMap<>();
        parameterMap.put("appCode", getOneValue(reqParameterMap, "appCode"));
        parameterMap.put("channelSerialNo", getOneValue(reqParameterMap, "channelSerialNo"));
        parameterMap.put("timestamp", getOneValue(reqParameterMap, "timestamp"));
        parameterMap.put("algorithm", getOneValue(reqParameterMap, "algorithm"));
        parameterMap.put("secretKey", getOneValue(reqParameterMap, "secretKey"));
        parameterMap.put("signature", getOneValue(reqParameterMap, "signature"));
        parameterMap.put("transCode", getOneValue(reqParameterMap, "transCode"));
        parameterMap.put("payload", getOneValue(reqParameterMap, "payload"));
        String transCode = parameterMap.get("transCode");
        String channelSerialNo = parameterMap.get("channelSerialNo");
        String serialNo = SnbSignUtil.getTranSerialNumber(smbConfiguration.getMerchantId(), 10);
        String respCode = SUCCESS_CODE;
        String respMsg = "成功";
        JSONObject jsonObject = null;
        logger.info("收到来自苏商银行的通知消息，channelSerialNo：{}, 通知类型：{}, 通知内容：{}", channelSerialNo, transCode, reqParameterMap);
        SMBRequestLogPO smbRequestLogPO = new SMBRequestLogPO();
        smbRequestLogPO.setSmbSerialNo(channelSerialNo);
        smbRequestLogPO.setTransCode(transCode);
        smbRequestLogPO.setChannelSerialNo(serialNo);
        try {
            Map map = SnbUtil.parseNotifyOrRespMsg(parameterMap);
            if ((boolean) map.get("verifyRes")) {
                String responsePayload = (String) map.get("payload");
                jsonObject = JSONObject.parseObject(responsePayload);
            } else {
                respCode = "00100004";
                respMsg = "签名验证错误";
                smbRequestLogPO.setRemark("签名验证错误");
            }
            smbRequestLogPO.setRequestPayload(JSONObject.toJSONString(map));
        } catch (Exception e) {
            respCode = "00100042";
            respMsg = "payload解密失败";
            smbRequestLogPO.setRemark("payload解密失败或者参数解析错误");
        }
        Map payloadMap = null;
        if (SUCCESS_CODE.equals(respCode)) {
            Function<Object, SMBNotifyRespDto> handler = smbSpiRegister.findHandler(transCode);
            if (handler != null) {
                payloadMap = new HashMap<>();
                SMBNotifyRespDto smbNotifyRespDto = null;
                if (TransCodeEnum.CREDIT_NOTIFY.getCode().equals(transCode)) {
                    smbNotifyRespDto = handler.apply(jsonObject.toJavaObject(CreditNotifyPayloadDto.class));
                } else if (TransCodeEnum.WITHDRAW_NOTIFY.getCode().equals(transCode)) {
                    smbNotifyRespDto = handler.apply(jsonObject.toJavaObject(WithdrawNotifyPayloadDto.class));
                } else {
                    smbNotifyRespDto = handler.apply(jsonObject);
                }
                if (!smbNotifyRespDto.getRespResult()) {
                    respCode = "00100046";
                    respMsg = smbNotifyRespDto.getRespMsg();
                    smbRequestLogPO.setRemark("业务处理返回失败：" + smbNotifyRespDto.getRespMsg());
                }
            } else {
                payloadMap = jsonObject;
            }
        }
        Map respMap = SnbUtil.getResponseNotifyContent(respCode, respMsg, serialNo, channelSerialNo, payloadMap);
        smbRequestLogPO.setResponsePayload(JSONObject.toJSONString(respMap));
        smbRequestLogService.add(smbRequestLogPO);
        return respMap;
    }

    private String getOneValue(Map<String, String[]> parameters, String name) {
        String[] value = parameters.get(name);
        return value == null ? "" : value[0];
    }
}
