package com.sunxung.factoring.start.impl;

import com.sunxung.factoring.component.util.ConstantUtil;
import com.sunxung.factoring.component.util.SpringUtil;
import com.sunxung.factoring.entity.financing.apply.CFinancingBasicInfo;
import com.sunxung.factoring.entity.smb.FinancingLoanInfo;
import com.sunxung.factoring.service.financing.apply.ICFinancingBasicInfoService;
import com.sunxung.factoring.service.financing.payment.IPaymentApplyService;
import com.sunxung.factoring.service.smb.IFinancingLoanInfoService;
import com.sunxung.factoring.service.smbApi.dto.SMBNotifyRespDto;
import com.sunxung.factoring.service.smbApi.dto.WithdrawNotifyPayloadDto;
import com.sunxung.factoring.service.smbApi.spi.SMBSpiRegister;

import java.util.function.Function;

/**
 * 苏商注册提款结果通知回调函数初始化
 */
public class SMBNoticeInit {

    public void run() {
        Function<WithdrawNotifyPayloadDto, SMBNotifyRespDto> withdrawNotifyHandler = withdrawNotifyPayloadDto -> {
            if (withdrawNotifyPayloadDto.getPayoutNo() != null) {
                FinancingLoanInfo loanInfo = getFinancingLoanInfoService().lambdaQuery().eq(FinancingLoanInfo::getPayoutNo, withdrawNotifyPayloadDto.getPayoutNo()).one();
                if (loanInfo != null) {
                    String beforeStatus = loanInfo.getPayoutStatus();
                    //借据编号
                    loanInfo.setDueBillNo(withdrawNotifyPayloadDto.getDuebillNo());
                    //到期日期
                    loanInfo.setMaturityDate(withdrawNotifyPayloadDto.getMaturity());
                    //提款申请日期
                    loanInfo.setLoanApplyDate(withdrawNotifyPayloadDto.getLoanApplyTime());
                    //放款日期
                    loanInfo.setPayoutDate(withdrawNotifyPayloadDto.getPayoutDate());
                    //放款状态
                    loanInfo.setPayoutStatus(withdrawNotifyPayloadDto.getStatus());
                    //放款金额
                    loanInfo.setLoanAmount(withdrawNotifyPayloadDto.getBusiAmt());
                    if ("03".equals(withdrawNotifyPayloadDto.getStatus())) {
                        //如果提款失败，添加失败原因
                        loanInfo.setFailMsg(withdrawNotifyPayloadDto.getRemark());
                    }
                    getFinancingLoanInfoService().updateById(loanInfo);
//                    if ("01".equals(beforeStatus) && "02".equals(withdrawNotifyPayloadDto.getStatus())) {
//                        //如果提款状态由处理中->成功、需要向sop推送融资信息
//                        if (loanInfo.getcFinancingBasicInfoId() != null) {
//                            CFinancingBasicInfo financingBasicInfo = getFinancingBasicInfoService().getById(loanInfo.getcFinancingBasicInfoId());
//                            if (financingBasicInfo != null) {
//                                getPaymentApplyService().financingToSop(financingBasicInfo, ConstantUtil.JIANGSU_SUSHANG_BANK, loanInfo.getPayoutDate());
//                            }
//                        }
//                    }
                }
            }
            SMBNotifyRespDto respDto = new SMBNotifyRespDto();
            respDto.setRespResult(true);
            return respDto;
        };
        getSMBSpiRegister().registerWithdrawNotifyHandler(withdrawNotifyHandler);
    }

    private static SMBSpiRegister smbSpiRegister;

    private static SMBSpiRegister getSMBSpiRegister() {
        if (smbSpiRegister == null) {
            smbSpiRegister = SpringUtil.getBeanByType(SMBSpiRegister.class);
        }
        return smbSpiRegister;
    }

    private static IFinancingLoanInfoService financingLoanInfoService;

    private static IFinancingLoanInfoService getFinancingLoanInfoService() {
        if (financingLoanInfoService == null) {
            financingLoanInfoService = SpringUtil.getBeanByType(IFinancingLoanInfoService.class);
        }
        return financingLoanInfoService;
    }

    private static ICFinancingBasicInfoService financingBasicInfoService;

    private static ICFinancingBasicInfoService getFinancingBasicInfoService() {
        if (financingBasicInfoService == null) {
            financingBasicInfoService = SpringUtil.getBeanByType(ICFinancingBasicInfoService.class);
        }
        return financingBasicInfoService;
    }

    private static IPaymentApplyService paymentApplyService;

    private static IPaymentApplyService getPaymentApplyService() {
        if (paymentApplyService == null) {
            paymentApplyService = SpringUtil.getBeanByType(IPaymentApplyService.class);
        }
        return paymentApplyService;
    }
}
