package com.sunxung.factoring.web.smb;


import com.sunxung.factoring.component.annotation.OperationLog;
import com.sunxung.factoring.component.enums.CreditInvestigationTypeEnums;
import com.sunxung.factoring.component.enums.OperationTypeEnum;
import com.sunxung.factoring.dict.impl.FileModuleDict;
import com.sunxung.factoring.entity.GridPage;
import com.sunxung.factoring.entity.ResponseJson;
import com.sunxung.factoring.entity.smb.FinancingFromInvestors;
import com.sunxung.factoring.entity.smb.FinancingFromInvestorsEletronicContract;
import com.sunxung.factoring.entity.smb.FinancingFromInvestorsNotice;
import com.sunxung.factoring.entity.smb.vo.FinancingFromInvestorsSearchVo;
import com.sunxung.factoring.entity.supplier.CreditInvestigationInfo;
import com.sunxung.factoring.entity.supplier.SupplierInfo;
import com.sunxung.factoring.entity.sys.FileStorageDO;
import com.sunxung.factoring.service.smb.IFinancingFromInvestorsService;
import com.sunxung.factoring.service.supplier.ICreditInvestigationInfoService;
import com.sunxung.factoring.service.sys.FileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * 资方融资管理
 *
 * @author liutao
 * @since 2024-08-26
 */
@RestController
@RequestMapping("/financing/from/investors")
public class FinancingFromInvestorsController {

    @Autowired
    private IFinancingFromInvestorsService financingFromInvestorsService;

    @Autowired
    private ICreditInvestigationInfoService creditInvestigationInfoService;

    @Autowired
    private FileService fileService;

    /**
     * 获取可以融资的供应商
     * @return
     */
    @RequestMapping("getSuppliers")
    public ResponseJson<List<SupplierInfo>> getSuppliers(){
        List<SupplierInfo> supplierInfos = financingFromInvestorsService.getSuppliers();
        return new ResponseJson(supplierInfos);
    }

    /**
     * 制作授信合同
     * @param supplierId
     * @return
     */
    @RequestMapping("makeContract")
    public ResponseJson<List<FinancingFromInvestorsEletronicContract>> makeContract(@RequestParam Long supplierId){
        List<FinancingFromInvestorsEletronicContract> financingFromInvestorsEletronicContracts = financingFromInvestorsService.makeContract(supplierId);
        return new ResponseJson(financingFromInvestorsEletronicContracts);
    }

    /**
     * 根据条件查询
     * @param financingFromInvestorsSearchVo
     * @return
     */
    @RequestMapping("findBySearch")
    public GridPage<FinancingFromInvestors> findBySearch(@RequestBody FinancingFromInvestorsSearchVo financingFromInvestorsSearchVo){
        return financingFromInvestorsService.findBySearch(financingFromInvestorsSearchVo);
    }

    /**
     * 新增/暂存/修改
     * @param financingFromInvestors
     * @return
     */
    @RequestMapping("add")
    @OperationLog(operationModule = "资方融资管理",operationType = OperationTypeEnum.INSERT,description = "新增/暂存/修改")
    public ResponseJson addInfo(@RequestBody FinancingFromInvestors financingFromInvestors){
        financingFromInvestorsService.addInfo(financingFromInvestors);
       return new ResponseJson();
    }

    /**
     * 查看详情
     * @param id
     * @return
     */
    @RequestMapping("getDetail")
    public ResponseJson<FinancingFromInvestors> getDetail(@RequestParam Long id){
        FinancingFromInvestors financingFromInvestors = financingFromInvestorsService.getDetail(id);
        return new ResponseJson(financingFromInvestors);
    }

    /**
     * 查看授信详情
     * @param id
     * @return
     */
    @RequestMapping("getCreditDetail")
    public ResponseJson<FinancingFromInvestorsNotice> getCreditDetail(@RequestParam Long id){
        FinancingFromInvestorsNotice financingFromInvestorsNotice = financingFromInvestorsService.getCreditDetail(id);
        return new ResponseJson<>(financingFromInvestorsNotice);
    }

    /**
     * 从苏商查询授信详情
     * @param id
     * @return
     */
    @RequestMapping("findCreditDetail")
    public ResponseJson<FinancingFromInvestorsNotice> findCreditDetail(@RequestParam Long id){
        FinancingFromInvestorsNotice financingFromInvestorsNotice = financingFromInvestorsService.findCreditDetail(id);
        return new ResponseJson<>(financingFromInvestorsNotice);
    }
}
