package com.sunxung.factoring.web.stakeholders;


import com.sunxung.factoring.component.annotation.OperationLog;
import com.sunxung.factoring.component.enums.OperationTypeEnum;
import com.sunxung.factoring.entity.BaseSearch;
import com.sunxung.factoring.entity.GridPage;
import com.sunxung.factoring.entity.ResponseJson;
import com.sunxung.factoring.entity.sys.stakeholders.SysStakeholdersManagement;
import com.sunxung.factoring.web.sys.stakeholders.ISysStakeholdersManagementService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 资方管理
 *
 * @author liutao
 * @since 2024-05-13
 */
@RestController
@RequestMapping("/sys/stakeholders/management")
public class SysStakeholdersManagementController {

    @Autowired
    private ISysStakeholdersManagementService stakeholdersManagementService;

    /**
     * 从sop同步资方信息
     *
     * @return
     */
    @RequestMapping("synchronization")
    @OperationLog(operationModule = "系统管理-资方配置", operationType = OperationTypeEnum.UPDATE, description = "从sop同步资方信息")
    public ResponseJson synchronizationDataFromSop() {
        stakeholdersManagementService.synchronizationDataFromSop();
        return new ResponseJson("同步资方成功！");
    }

    /**
     * 根据条件查询资方信息
     *
     * @param baseSearch
     * @return
     */
    @RequestMapping("search")
    public GridPage<SysStakeholdersManagement> findBySearch(@RequestBody BaseSearch baseSearch) {
        return stakeholdersManagementService.findBySearch(baseSearch);
    }

    /**
     * 关键字模糊查询
     *
     * @param keyword
     * @return
     */
    @GetMapping("findByKeyword")
    public ResponseJson<SysStakeholdersManagement> findByKeyword(String keyword) {
        return new ResponseJson(stakeholdersManagementService.findByKeyword(keyword));
    }
}
