package com.sunxung.factoring.web.sys.stakeholders.impl;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.sunxung.factoring.component.util.JsonUtil;
import com.sunxung.factoring.entity.BaseSearch;
import com.sunxung.factoring.entity.GridPage;
import com.sunxung.factoring.entity.ResponseJson;
import com.sunxung.factoring.entity.sys.Dictionary;
import com.sunxung.factoring.entity.sys.stakeholders.SysStakeholdersManagement;
import com.sunxung.factoring.mapper.sys.stakeholders.SysStakeholdersManagementMapper;
import com.sunxung.factoring.service.sys.DictionaryService;
import com.sunxung.factoring.web.sys.stakeholders.ISysStakeholdersManagementService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.util.List;

/**
 * <p>
 * 资方管理 服务实现类
 * </p>
 *
 * @author liutao
 * @since 2024-05-13
 */
@Service
public class SysStakeholdersManagementServiceImpl extends ServiceImpl<SysStakeholdersManagementMapper, SysStakeholdersManagement> implements ISysStakeholdersManagementService {

    @Autowired
    private RestTemplate restTemplate;

    @Value("${sop.stakeholdersUrl}")
    private String synchronizationDataFromSopUrl;

    @Autowired
    private DictionaryService dictionaryService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void synchronizationDataFromSop() {
        //本地记录
        List<SysStakeholdersManagement> list = this.list();
        //sop记录
        ResponseJson forObject = restTemplate.getForObject(synchronizationDataFromSopUrl, ResponseJson.class);
        JSONArray jsonArray = new JSONArray(forObject.getData());
        //本地不存在的记录insert到本地
        for (Object o : jsonArray) {
            SysStakeholdersManagement sysStakeholdersManagement = JsonUtil.parseObject(o.toString(), SysStakeholdersManagement.class);
            JSONObject jsonObject = new JSONObject(o);
            Long id = jsonObject.getLong("id");
            sysStakeholdersManagement.setId(id);
            boolean anyMatch = list.stream().anyMatch(r -> r.getId().equals(sysStakeholdersManagement.getId()));
            if (!anyMatch) {
                //没有就新增
                save(sysStakeholdersManagement);
            } else {
                //有就更新
                updateById(sysStakeholdersManagement);
            }
        }
    }

    @Override
    public GridPage<SysStakeholdersManagement> findBySearch(BaseSearch baseSearch) {
        Page page = PageHelper.startPage(baseSearch.getPage(), baseSearch.getRows());
        List<SysStakeholdersManagement> list = this.lambdaQuery().eq(SysStakeholdersManagement::getInstitutionName,baseSearch.getFastSearch()).list();
        for (SysStakeholdersManagement sysStakeholdersManagement : list) {
            String institutionType = sysStakeholdersManagement.getInstitutionType();
            Dictionary dictionary = dictionaryService.getByCode(institutionType);
            sysStakeholdersManagement.setInstitutionTypeName(dictionary.getName());
        }
        return new GridPage<>(baseSearch.getRows(), baseSearch.getPage(), page.getTotal(), list);
    }

    @Override
    public List<SysStakeholdersManagement> findByKeyword(String keyword) {
        QueryWrapper<SysStakeholdersManagement> wrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(keyword)) {
            wrapper.like("institution_name", "%" + keyword + "%");
        }
        return baseMapper.selectList(wrapper);
    }
}
